@@ -115,63 +115,46 @@ bool CppCheckExecutor::parseFromArgs(CppCheck *cppcheck, int argc, const char* c
             if (FileLister::isDirectory(path))
                 ++iter;
             else {
-                // If the include path is not found, warn user (unless --quiet
-                // was used) and remove the non-existing path from the list.
-                if (!settings._errorsOnly)
-                    std::cout << "cppcheck: warning: Couldn't find path given by -I '" << path << '\'' << std::endl;
+                // If the include path is not found, warn user and remove the non-existing path from the list.
+                std::cout << "cppcheck: warning: Couldn't find path given by -I '" << path << '\'' << std::endl;
                 iter = settings._includePaths.erase(iter);
             }
         }
     }
 
-    const std::vector<std::string>& pathnames = parser.GetPathNames();
-
-    if (!pathnames.empty()) {
-        // Execute recursiveAddFiles() to each given file parameter
-        std::vector<std::string>::const_iterator iter;
-        for (iter = pathnames.begin(); iter != pathnames.end(); ++iter)
-            FileLister::recursiveAddFiles(_files, Path::toNativeSeparators(*iter), _settings->library.markupExtensions());
+    // Output a warning for the user if he tries to exclude headers
+    bool warn = false;
+    const std::vector<std::string>& ignored = parser.GetIgnoredPaths();
+    for (std::vector<std::string>::const_iterator i = ignored.cbegin(); i != ignored.cend(); ++i) {
+        if (Path::isHeader(*i)) {
+            warn = true;
+            break;
+        }
+    }
+    if (warn) {
+        std::cout << "cppcheck: filename exclusion does not apply to header (.h and .hpp) files." << std::endl;
+        std::cout << "cppcheck: Please use --suppress for ignoring results from the header files." << std::endl;
     }
 
-    if (!_files.empty()) {
-        // Remove header files from the list of ignored files.
-        // Also output a warning for the user.
-        // TODO: Remove all unknown files? (use FileLister::acceptFile())
-        bool warn = false;
-        std::vector<std::string> ignored = parser.GetIgnoredPaths();
-        for (std::vector<std::string>::iterator i = ignored.begin(); i != ignored.end();) {
-            const std::string extension = Path::getFilenameExtension(*i);
-            if (extension == ".h" || extension == ".hpp") {
-                i = ignored.erase(i);
-                warn = true;
-            } else
-                ++i;
-        }
-        if (warn) {
-            std::cout << "cppcheck: filename exclusion does not apply to header (.h and .hpp) files." << std::endl;
-            std::cout << "cppcheck: Please use --suppress for ignoring results from the header files." << std::endl;
-        }
+    const std::vector<std::string>& pathnames = parser.GetPathNames();
 
 #if defined(_WIN32)
-        // For Windows we want case-insensitive path matching
-        const bool caseSensitive = false;
+    // For Windows we want case-insensitive path matching
+    const bool caseSensitive = false;
 #else
-        const bool caseSensitive = true;
+    const bool caseSensitive = true;
 #endif
-        PathMatch matcher(parser.GetIgnoredPaths(), caseSensitive);
-        for (std::map<std::string, std::size_t>::iterator i = _files.begin(); i != _files.end();) {
-            if (matcher.Match(i->first))
-                _files.erase(i++);
-            else
-                ++i;
-        }
-    } else {
-        std::cout << "cppcheck: error: could not find or open any of the paths given." << std::endl;
-        return false;
+    if (!pathnames.empty()) {
+        // Execute recursiveAddFiles() to each given file parameter
+        PathMatch matcher(ignored, caseSensitive);
+        for (std::vector<std::string>::const_iterator iter = pathnames.begin(); iter != pathnames.end(); ++iter)
+            FileLister::recursiveAddFiles(_files, Path::toNativeSeparators(*iter), _settings->library.markupExtensions(), matcher);
     }
 
     if (_files.empty()) {
-        std::cout << "cppcheck: error: no files to check - all paths ignored." << std::endl;
+        std::cout << "cppcheck: error: could not find or open any of the paths given." << std::endl;
+        if (!ignored.empty())
+            std::cout << "cppcheck: Maybe all paths were ignored?" << std::endl;
         return false;
     }
     return true;
@@ -508,7 +491,7 @@ static bool loadDbgHelp()
 {
     hLibDbgHelp = ::LoadLibraryW(L"Dbghelp.dll");
     if (!hLibDbgHelp)
-        return true;
+        return false;
     pStackWalk64 = (fpStackWalk64) ::GetProcAddress(hLibDbgHelp, "StackWalk64");
     pSymGetModuleBase64 = (fpSymGetModuleBase64) ::GetProcAddress(hLibDbgHelp, "SymGetModuleBase64");
     pSymGetSymFromAddr64 = (fpSymGetSymFromAddr64) ::GetProcAddress(hLibDbgHelp, "SymGetSymFromAddr64");
@@ -583,7 +566,7 @@ static void PrintCallstack(FILE* f, PEXCEPTION_POINTERS ex)
                 "%lu. 0x%08I64X in ",
                 frame, (ULONG64)stack.AddrPC.Offset);
         fputs((const char *)undname, f);
-        fputs("\n", f);
+        fputc('\n', f);
         if (0==stack.AddrReturn.Offset || beyond_main>2) // StackWalk64() sometimes doesn't reach any end...
             break;
     }
@@ -595,21 +578,22 @@ static void PrintCallstack(FILE* f, PEXCEPTION_POINTERS ex)
 static void writeMemoryErrorDetails(FILE* f, PEXCEPTION_POINTERS ex, const char* description)
 {
     fputs(description, f);
+    fprintf(f, " (instruction: 0x%p) ", ex->ExceptionRecord->ExceptionAddress);
     // Using %p for ULONG_PTR later on, so it must have size identical to size of pointer
     // This is not the universally portable solution but good enough for Win32/64
     C_ASSERT(sizeof(void*) == sizeof(ex->ExceptionRecord->ExceptionInformation[1]));
     switch (ex->ExceptionRecord->ExceptionInformation[0]) {
     case 0:
-        fprintf(f, " reading from 0x%p",
-                ex->ExceptionRecord->ExceptionInformation[1]);
+        fprintf(f, "reading from 0x%p",
+                reinterpret_cast<void*>(ex->ExceptionRecord->ExceptionInformation[1]));
         break;
     case 1:
-        fprintf(f, " writing at 0x%p",
-                ex->ExceptionRecord->ExceptionInformation[1]);
+        fprintf(f, "writing to 0x%p",
+                reinterpret_cast<void*>(ex->ExceptionRecord->ExceptionInformation[1]));
         break;
     case 8:
-        fprintf(f, " data execution prevention at 0x%p",
-                ex->ExceptionRecord->ExceptionInformation[1]);
+        fprintf(f, "data execution prevention at 0x%p",
+                reinterpret_cast<void*>(ex->ExceptionRecord->ExceptionInformation[1]));
         break;
     default:
         break;
@@ -695,7 +679,7 @@ static int filterException(int code, PEXCEPTION_POINTERS ex)
                 code);
         break;
     }
-    fputs("\n", f);
+    fputc('\n', f);
     PrintCallstack(f, ex);
     fflush(f);
     return EXCEPTION_EXECUTE_HANDLER;
@@ -804,7 +788,7 @@ int CppCheckExecutor::check_internal(CppCheck& cppcheck, int /*argc*/, const cha
                 || !_settings->library.processMarkupAfterCode(i->first)) {
                 returnValue += cppcheck.check(i->first);
                 processedsize += i->second;
-                if (!settings._errorsOnly)
+                if (!settings.quiet)
                     reportStatus(c + 1, _files.size(), processedsize, totalfilesize);
                 c++;
             }
@@ -816,7 +800,7 @@ int CppCheckExecutor::check_internal(CppCheck& cppcheck, int /*argc*/, const cha
             if (_settings->library.markupFile(i->first) && _settings->library.processMarkupAfterCode(i->first)) {
                 returnValue += cppcheck.check(i->first);
                 processedsize += i->second;
-                if (!settings._errorsOnly)
+                if (!settings.quiet)
                     reportStatus(c + 1, _files.size(), processedsize, totalfilesize);
                 c++;
             }
