@@ -44,6 +44,7 @@
 #include "logview.h"
 #include "filelist.h"
 #include "showtypes.h"
+//#include "librarydialog.h"
 
 static const QString OnlineHelpURL("http://cppcheck.sourceforge.net/manual.html");
 
@@ -77,6 +78,8 @@ MainWindow::MainWindow(TranslationHandler* th, QSettings* settings) :
     connect(mLineEditFilter, SIGNAL(textChanged(const QString&)), mFilterTimer, SLOT(start()));
     connect(mLineEditFilter, SIGNAL(returnPressed()), this, SLOT(FilterResults()));
 
+    connect(mUI.mActionPrint, SIGNAL(triggered()), mUI.mResults, SLOT(Print()));
+    connect(mUI.mActionPrintPreview, SIGNAL(triggered()), mUI.mResults, SLOT(PrintPreview()));
     connect(mUI.mActionQuit, SIGNAL(triggered()), this, SLOT(close()));
     connect(mUI.mActionCheckFiles, SIGNAL(triggered()), this, SLOT(CheckFiles()));
     connect(mUI.mActionCheckDirectory, SIGNAL(triggered()), this, SLOT(CheckDirectory()));
@@ -97,6 +100,7 @@ MainWindow::MainWindow(TranslationHandler* th, QSettings* settings) :
     connect(mUI.mActionShowHidden, SIGNAL(triggered()), mUI.mResults, SLOT(ShowHiddenResults()));
     connect(mUI.mActionViewLog, SIGNAL(triggered()), this, SLOT(ShowLogView()));
     connect(mUI.mActionViewStats, SIGNAL(triggered()), this, SLOT(ShowStatistics()));
+    connect(mUI.mActionLibraryEditor, SIGNAL(triggered()), this, SLOT(ShowLibraryEditor()));
 
     connect(mUI.mActionRecheck, SIGNAL(triggered()), this, SLOT(ReCheck()));
 
@@ -134,6 +138,9 @@ MainWindow::MainWindow(TranslationHandler* th, QSettings* settings) :
 
     EnableCheckButtons(true);
 
+    mUI.mActionPrint->setShortcut(QKeySequence::Print);
+    mUI.mActionPrint->setEnabled(false);
+    mUI.mActionPrintPreview->setEnabled(false);
     mUI.mActionClearResults->setEnabled(false);
     mUI.mActionSave->setEnabled(false);
     mUI.mActionRecheck->setEnabled(false);
@@ -642,7 +649,7 @@ Settings MainWindow::GetCppcheckSettings()
     result.addEnabled("missingInclude");
     result.debug = false;
     result.debugwarnings = mSettings->value(SETTINGS_SHOW_DEBUG_WARNINGS, false).toBool();
-    result._errorsOnly = false;
+    result.quiet = false;
     result._verbose = true;
     result._force = mSettings->value(SETTINGS_CHECK_FORCE, 1).toBool();
     result._xml = false;
@@ -695,6 +702,8 @@ void MainWindow::CheckDone()
     if (mUI.mResults->HasResults()) {
         mUI.mActionClearResults->setEnabled(true);
         mUI.mActionSave->setEnabled(true);
+        mUI.mActionPrint->setEnabled(true);
+        mUI.mActionPrintPreview->setEnabled(true);
     }
 
     for (int i = 0; i < MaxRecentProjects + 1; i++) {
@@ -704,6 +713,7 @@ void MainWindow::CheckDone()
 
     // Notify user - if the window is not active - that check is ready
     QApplication::alert(this, 3000);
+    ShowStatistics();
 }
 
 void MainWindow::CheckLockDownUI()
@@ -768,6 +778,8 @@ void MainWindow::ClearResults()
     mUI.mResults->Clear(true);
     mUI.mActionClearResults->setEnabled(false);
     mUI.mActionSave->setEnabled(false);
+    mUI.mActionPrint->setEnabled(false);
+    mUI.mActionPrintPreview->setEnabled(false);
 }
 
 void MainWindow::OpenResults()
@@ -1103,6 +1115,8 @@ void MainWindow::CheckProject(Project *project)
     // file's location directory as root path
     if (rootpath.isEmpty() || rootpath == ".")
         mCurrentDirectory = inf.canonicalPath();
+    else if (rootpath.startsWith("."))
+        mCurrentDirectory = inf.canonicalPath() + rootpath.mid(1);
     else
         mCurrentDirectory = rootpath;
 
@@ -1203,6 +1217,12 @@ void MainWindow::ShowStatistics()
     statsDialog.exec();
 }
 
+void MainWindow::ShowLibraryEditor()
+{
+    //LibraryDialog libraryDialog(this);
+    //libraryDialog.exec();
+}
+
 void MainWindow::Log(const QString &logline)
 {
     if (mLogView) {
