@@ -223,6 +223,11 @@ protected slots:
     */
     void HideAllIdResult();
 
+    /**
+    * @brief Slot for context menu item to open the folder containing the current file.
+    */
+    void OpenContainingFolder();
+
     /**
     * @brief Slot for selection change in the results tree.
     *
@@ -285,7 +290,15 @@ protected:
     * @param target Error tree item to open
     * @param fullPath Are we copying full path or only filename?
     */
-    void CopyPath(QStandardItem *target, bool fullPath);
+    void CopyPathToClipboard(QStandardItem *target, bool fullPath);
+
+    /**
+    * @brief Helper function returning the filename/full path of the error tree item \a target.
+    *
+    * @param target The error tree item containing the filename/full path
+    * @param fullPath Whether or not to retrieve the full path or only the filename.
+    */
+    QString GetFilePath(QStandardItem *target, bool fullPath);
 
     /**
     * @brief Context menu event (user right clicked on the tree)
