@@ -29,6 +29,7 @@
 class ErrorItem;
 class ApplicationList;
 class QModelIndex;
+class QPrinter;
 class QSettings;
 class CheckStatistics;
 
@@ -222,6 +223,22 @@ public slots:
     */
     void UpdateDetails(const QModelIndex &index);
 
+    /**
+    * @brief Slot opening a print dialog to print the current report
+    */
+    void Print();
+
+    /**
+    * @brief Slot printing the current report to the printer.
+    * @param printer The printer used for printing the report.
+    */
+    void Print(QPrinter* printer);
+
+    /**
+    * @brief Slot opening a print preview dialog
+    */
+    void PrintPreview();
+
 protected:
     /**
     * @brief Have any errors been found
