@@ -31,6 +31,7 @@
 #include <list>
 #include <cassert>     // <- assert
 #include <cstdlib>
+#include <stack>
 
 //---------------------------------------------------------------------------
 
@@ -64,20 +65,6 @@ void CheckBufferOverrun::arrayIndexOutOfBoundsError(const Token *tok, const Arra
 
 void CheckBufferOverrun::arrayIndexOutOfBoundsError(const Token *tok, const ArrayInfo &arrayInfo, const std::vector<ValueFlow::Value> &index)
 {
-    std::ostringstream errmsg;
-
-    errmsg << "Array '" << arrayInfo.varname();
-    for (std::size_t i = 0; i < arrayInfo.num().size(); ++i)
-        errmsg << "[" << arrayInfo.num(i) << "]";
-    if (index.size() == 1)
-        errmsg << "' accessed at index " << index[0].intvalue << ", which is out of bounds.";
-    else {
-        errmsg << "' index " << arrayInfo.varname();
-        for (std::size_t i = 0; i < index.size(); ++i)
-            errmsg << "[" << index[i].intvalue << "]";
-        errmsg << " out of bounds.";
-    }
-
     const Token *condition = nullptr;
     for (std::size_t i = 0; i < index.size(); ++i) {
         if (condition == nullptr)
@@ -87,12 +74,38 @@ void CheckBufferOverrun::arrayIndexOutOfBoundsError(const Token *tok, const Arra
     if (condition != nullptr) {
         if (!_settings->isEnabled("warning"))
             return;
-        errmsg << " Otherwise condition '" << condition->expressionString() << "' is redundant.";
+
+        std::ostringstream errmsg;
+        errmsg << ValueFlow::eitherTheConditionIsRedundant(condition) << " or the array '" << arrayInfo.varname();
+        for (std::size_t i = 0; i < arrayInfo.num().size(); ++i)
+            errmsg << "[" << arrayInfo.num(i) << "]";
+        if (index.size() == 1)
+            errmsg << "' is accessed at index " << index[0].intvalue << ", which is out of bounds.";
+        else {
+            errmsg << "' index " << arrayInfo.varname();
+            for (std::size_t i = 0; i < index.size(); ++i)
+                errmsg << "[" << index[i].intvalue << "]";
+            errmsg << " is out of bounds.";
+        }
+
         std::list<const Token *> callstack;
         callstack.push_back(tok);
         callstack.push_back(condition);
         reportError(callstack, Severity::warning, "arrayIndexOutOfBoundsCond", errmsg.str(), 0U, false);
     } else {
+        std::ostringstream errmsg;
+        errmsg << "Array '" << arrayInfo.varname();
+        for (std::size_t i = 0; i < arrayInfo.num().size(); ++i)
+            errmsg << "[" << arrayInfo.num(i) << "]";
+        if (index.size() == 1)
+            errmsg << "' accessed at index " << index[0].intvalue << ", which is out of bounds.";
+        else {
+            errmsg << "' index " << arrayInfo.varname();
+            for (std::size_t i = 0; i < index.size(); ++i)
+                errmsg << "[" << index[i].intvalue << "]";
+            errmsg << " out of bounds.";
+        }
+
         reportError(tok, Severity::error, "arrayIndexOutOfBounds", errmsg.str());
     }
 }
@@ -306,7 +319,7 @@ static bool checkMinSizes(const std::list<Library::ArgumentChecks::MinSize> &min
                 const MathLib::bigint sz = MathLib::toLongNumber(argtok->str());
                 if ((std::size_t)sz > arraySize)
                     error = true;
-            } else if (argtok->type() == Token::eChar && Token::Match(argtok->next(), ",|)") && charSizeToken)
+            } else if (argtok->tokType() == Token::eChar && Token::Match(argtok->next(), ",|)") && charSizeToken)
                 *charSizeToken = argtok; //sizeArgumentAsCharError(argtok);
             break;
         case Library::ArgumentChecks::MinSize::MUL:
@@ -339,7 +352,7 @@ static bool checkMinSizes(const std::list<Library::ArgumentChecks::MinSize> &min
         }
         break;
         case Library::ArgumentChecks::MinSize::SIZEOF:
-            if (argtok->type() == Token::eString && Token::getStrLength(argtok) >= arraySize)
+            if (argtok->tokType() == Token::eString && Token::getStrLength(argtok) >= arraySize)
                 error = true;
             break;
         case Library::ArgumentChecks::MinSize::NONE:
@@ -361,6 +374,10 @@ void CheckBufferOverrun::checkFunctionParameter(const Token &ftok, unsigned int
         for (std::size_t i = 0; i < arrayInfo.num().size(); ++i)
             arraySize *= arrayInfo.num(i);
 
+        // dimension is 0 or unknown => bailout
+        if (arraySize == 0)
+            return;
+
         const Token *charSizeToken = nullptr;
         if (checkMinSizes(*minsizes, &ftok, (std::size_t)arraySize, &charSizeToken, _settings))
             bufferOverrunError(callstack, arrayInfo.varname());
@@ -454,10 +471,10 @@ void CheckBufferOverrun::checkFunctionParameter(const Token &ftok, unsigned int
                 MathLib::bigint argsize = _tokenizer->sizeOfType(argument->typeStartToken());
                 if (argsize == 100) // unknown size
                     argsize = 0;
-                while (Token::Match(tok2, "[ %num% ] [,)[]")) {
+                do {
                     argsize *= MathLib::toLongNumber(tok2->strAt(1));
                     tok2 = tok2->tokAt(3);
-                }
+                } while (Token::Match(tok2, "[ %num% ] [,)[]"));
 
                 MathLib::bigint arraysize = arrayInfo.element_size();
                 if (arraysize == 100) // unknown size
@@ -531,7 +548,7 @@ void CheckBufferOverrun::checkScope(const Token *tok, const std::vector<std::str
     if ((declarationId > 0 && Token::Match(tok, "%varid% [", declarationId)) ||
         (declarationId == 0 && Token::Match(tok, (varnames + " [").c_str()))) {
 
-        const Token *tok2 = tok;
+        const Token *tok2 = tok->next();
         while (tok2->str() != "[")
             tok2 = tok2->next();
         valueFlowCheckArrayIndex(tok2, arrayInfo);
@@ -657,23 +674,25 @@ void CheckBufferOverrun::checkScope(const Token *tok, const std::vector<std::str
 
         if (total_size > 0) {
             // Writing data into array..
-            if ((declarationId > 0 && Token::Match(tok, "strcpy|strcat ( %varid% , %str% )", declarationId)) ||
-                (declarationId == 0 && Token::Match(tok, ("strcpy|strcat ( " + varnames + " , %str% )").c_str()))) {
-                const std::size_t len = Token::getStrLength(tok->tokAt(varcount + 4));
-                if (len >= (unsigned int)total_size) {
-                    bufferOverrunError(tok, declarationId > 0 ? emptyString : varnames);
-                    continue;
-                }
-            } else if ((declarationId > 0 && Token::Match(tok, "strcpy|strcat ( %varid% , %var% )", declarationId)) ||
-                       (declarationId == 0 && Token::Match(tok, ("strcpy|strcat ( " + varnames + " , %var% )").c_str()))) {
-                const Variable *var = tok->tokAt(varcount + 4)->variable();
-                if (var && var->isArray() && var->dimensions().size() == 1) {
-                    const MathLib::bigint len = var->dimension(0);
-                    if (len > total_size) {
-                        if (printInconclusive)
-                            possibleBufferOverrunError(tok, tok->strAt(4), tok->strAt(2), tok->str() == "strcat");
+            if ((declarationId > 0 && Token::Match(tok, "strcpy|strcat ( %varid% , %str%|%var% )", declarationId)) ||
+                (declarationId == 0 && Token::Match(tok, ("strcpy|strcat ( " + varnames + " , %str%|%var% )").c_str()))) {
+                const Token* lastParamTok = tok->tokAt(varcount + 4);
+                if (lastParamTok->tokType() == Token::Type::eString) {
+                    const std::size_t len = Token::getStrLength(lastParamTok);
+                    if (len >= (unsigned int)total_size) {
+                        bufferOverrunError(tok, declarationId > 0 ? emptyString : varnames);
                         continue;
                     }
+                } else {
+                    const Variable *var = lastParamTok->variable();
+                    if (var && var->isArray() && var->dimensions().size() == 1) {
+                        const MathLib::bigint len = var->dimension(0);
+                        if (len > total_size) {
+                            if (printInconclusive)
+                                possibleBufferOverrunError(tok, tok->strAt(4), tok->strAt(2), tok->str() == "strcat");
+                            continue;
+                        }
+                    }
                 }
             }
 
@@ -683,14 +702,14 @@ void CheckBufferOverrun::checkScope(const Token *tok, const std::vector<std::str
                 std::size_t charactersAppend = 0;
                 const Token *tok2 = tok;
 
-                while (Token::Match(tok2, strcatPattern.c_str(), declarationId)) {
+                do {
                     charactersAppend += Token::getStrLength(tok2->tokAt(4 + varcount));
                     if (charactersAppend >= static_cast<std::size_t>(total_size)) {
                         bufferOverrunError(tok2);
                         break;
                     }
                     tok2 = tok2->tokAt(7 + varcount);
-                }
+                } while (Token::Match(tok2, strcatPattern.c_str(), declarationId));
             }
 
             // Check function call..
@@ -957,7 +976,7 @@ void CheckBufferOverrun::checkScope(const Token *tok, const ArrayInfo &arrayInfo
                 std::size_t charactersAppend = 0;
                 const Token *tok2 = tok;
 
-                while (tok2 && Token::Match(tok2, "strcat ( %varid% , %str% ) ;", declarationId)) {
+                while (Token::Match(tok2, "strcat ( %varid% , %str% ) ;", declarationId)) {
                     charactersAppend += Token::getStrLength(tok2->tokAt(4));
                     if (charactersAppend >= (unsigned int)total_size) {
                         bufferOverrunError(tok2, arrayInfo.varname());
@@ -970,6 +989,59 @@ void CheckBufferOverrun::checkScope(const Token *tok, const ArrayInfo &arrayInfo
     }
 }
 
+//---------------------------------------------------------------------------
+// Negative size in array declarations
+//---------------------------------------------------------------------------
+
+static bool isVLAIndex(const Token *index)
+{
+    std::stack<const Token *> tokens;
+    tokens.push(index);
+    while (!tokens.empty()) {
+        const Token *tok = tokens.top();
+        tokens.pop();
+        if (!tok)
+            continue;
+        if (tok->varId() != 0U)
+            return true;
+        if (tok->str() == "?") {
+            // this is a VLA index if both expressions around the ":" is VLA index
+            if (tok->astOperand2() &&
+                tok->astOperand2()->str() == ":" &&
+                isVLAIndex(tok->astOperand2()->astOperand1()) &&
+                isVLAIndex(tok->astOperand2()->astOperand2()))
+                return true;
+            continue;
+        }
+        tokens.push(tok->astOperand1());
+        tokens.push(tok->astOperand2());
+    }
+    return false;
+}
+
+void CheckBufferOverrun::negativeArraySize()
+{
+    const SymbolDatabase* symbolDatabase = _tokenizer->getSymbolDatabase();
+    for (unsigned int i = 1; i <= _tokenizer->varIdCount(); i++) {
+        const Variable * const var = symbolDatabase->getVariableFromVarId(i);
+        if (!var || !var->isArray())
+            continue;
+        const Token * const nameToken = var->nameToken();
+        if (!Token::Match(nameToken, "%var% [") || !nameToken->next()->astOperand2())
+            continue;
+        const ValueFlow::Value *sz = nameToken->next()->astOperand2()->getValueLE(-1,_settings);
+        // don't warn about constant negative index because that is a compiler error
+        if (sz && isVLAIndex(nameToken->next()->astOperand2()))
+            negativeArraySizeError(nameToken);
+    }
+}
+
+void CheckBufferOverrun::negativeArraySizeError(const Token *tok)
+{
+    reportError(tok, Severity::error, "negativeArraySize",
+                "Declaration of array '" + (tok ? tok->str() : std::string()) + "' with negative size is undefined behaviour");
+}
+
 //---------------------------------------------------------------------------
 // Checking member variables of structs.
 //---------------------------------------------------------------------------
@@ -1015,7 +1087,7 @@ void CheckBufferOverrun::checkGlobalAndLocalVariable()
         const Variable * const var = symbolDatabase->getVariableFromVarId(i);
         if (var && var->isArray() && var->dimension(0) > 0) {
             const Token *tok = var->nameToken();
-            while (tok && tok->str() != ";") {
+            do {
                 if (tok->str() == "{") {
                     if (Token::simpleMatch(tok->previous(), "= {"))
                         tok = tok->link();
@@ -1023,7 +1095,7 @@ void CheckBufferOverrun::checkGlobalAndLocalVariable()
                         break;
                 }
                 tok = tok->next();
-            }
+            } while (tok && tok->str() != ";");
             if (!tok)
                 break;
             if (tok->str() == "{")
@@ -1059,7 +1131,7 @@ void CheckBufferOverrun::checkGlobalAndLocalVariable()
                                          "Check (BufferOverrun::checkGlobalAndLocalVariable)",
                                          tok->progressValue());
 
-            if (Token::Match(tok, "[*;{}] %var% = new %type% [ %num% ]")) {
+            if (_tokenizer->isCPP() && Token::Match(tok, "[*;{}] %var% = new %type% [ %num% ]")) {
                 size = MathLib::toLongNumber(tok->strAt(6));
                 type = tok->strAt(4);
                 var = tok->next()->variable();
@@ -1067,7 +1139,7 @@ void CheckBufferOverrun::checkGlobalAndLocalVariable()
                 if (size < 0) {
                     negativeMemoryAllocationSizeError(tok->next()->next());
                 }
-            } else if (Token::Match(tok, "[*;{}] %var% = new %type% ( %num%|%name% )")) {
+            } else if (_tokenizer->isCPP() && Token::Match(tok, "[*;{}] %var% = new %type% ( %num%|%name% )")) {
                 size = 1;
                 type = tok->strAt(4);
                 var = tok->next()->variable();
@@ -1386,6 +1458,8 @@ MathLib::biguint CheckBufferOverrun::countSprintfLength(const std::string &input
             case 'X':
             case 'i':
                 i_d_x_f_found = true;
+                handleNextParameter = true;
+                break;
             case 'c':
             case 'e':
             case 'E':
@@ -1398,13 +1472,13 @@ MathLib::biguint CheckBufferOverrun::countSprintfLength(const std::string &input
                 break;
             case 'd':
                 i_d_x_f_found = true;
-                if (paramIter != parameters.end() && *paramIter && (*paramIter)->type() != Token::eString)
+                if (paramIter != parameters.end() && *paramIter && (*paramIter)->tokType() != Token::eString)
                     parameterLength = (*paramIter)->str().length();
 
                 handleNextParameter = true;
                 break;
             case 's':
-                if (paramIter != parameters.end() && *paramIter && (*paramIter)->type() == Token::eString)
+                if (paramIter != parameters.end() && *paramIter && (*paramIter)->tokType() == Token::eString)
                     parameterLength = Token::getStrLength(*paramIter);
 
                 handleNextParameter = true;
@@ -1487,7 +1561,7 @@ void CheckBufferOverrun::checkBufferAllocatedWithStrlen()
                 dstVarId = tok->varId();
                 srcVarId = tok->tokAt(6)->varId();
                 tok      = tok->tokAt(8);
-            } else if (Token::Match(tok, "%var% = new char [ strlen ( %name% ) ]")) {
+            } else if (_tokenizer->isCPP() && Token::Match(tok, "%var% = new char [ strlen ( %name% ) ]")) {
                 dstVarId = tok->varId();
                 srcVarId = tok->tokAt(7)->varId();
                 tok      = tok->tokAt(9);
@@ -1533,6 +1607,8 @@ void CheckBufferOverrun::checkStringArgument()
             for (const Token *argtok = tok->tokAt(2); argtok; argtok = argtok->nextArgument(), argnr++) {
                 if (!Token::Match(argtok, "%name%|%str% ,|)"))
                     continue;
+                if (argtok->variable() && !argtok->variable()->isPointer())
+                    continue;
                 const Token *strtoken = argtok->getValueTokenMinStrSize();
                 if (!strtoken)
                     continue;
@@ -1691,9 +1767,11 @@ void CheckBufferOverrun::arrayIndexThenCheck()
                     return;
 
                 // skip comparison
-                if (tok->type() == Token::eComparisonOp)
+                if (tok->tokType() == Token::eComparisonOp)
                     tok = tok->tokAt(2);
 
+                if (!tok)
+                    break;
                 // skip close parentheses
                 if (tok->str() == ")")
                     tok = tok->next();
@@ -1764,7 +1842,7 @@ Check::FileInfo* CheckBufferOverrun::getFileInfo(const Tokenizer *tokenizer, con
     return fileInfo;
 }
 
-void CheckBufferOverrun::analyseWholeProgram(const std::list<Check::FileInfo*> &fileInfo, ErrorLogger &errorLogger)
+void CheckBufferOverrun::analyseWholeProgram(const std::list<Check::FileInfo*> &fileInfo, const Settings&, ErrorLogger &errorLogger)
 {
     // Merge all fileInfo
     MyFileInfo all;
