@@ -78,7 +78,7 @@ void CheckSizeof::checkSizeofForArrayParameter()
                 }
 
                 const Variable *var = varTok->variable();
-                if (var && var->isArray() && var->isArgument())
+                if (var && var->isArray() && var->isArgument() && !var->isReference())
                     sizeofForArrayParameterError(tok);
             }
         }
@@ -251,7 +251,7 @@ void CheckSizeof::sizeofCalculation()
     for (const Token *tok = _tokenizer->tokens(); tok; tok = tok->next()) {
         if (Token::simpleMatch(tok, "sizeof (")) {
             const Token *argument = tok->next()->astOperand2();
-            if (argument && argument->isCalculation() && (!argument->isExpandedMacro() || printInconclusive))
+            if (argument && argument->isCalculation(true) && (!argument->isExpandedMacro() || printInconclusive))
                 sizeofCalculationError(argument, argument->isExpandedMacro());
         }
     }
