@@ -17,7 +17,6 @@
  */
 
 #include "checkstl.h"
-#include "executionpath.h"
 #include "symboldatabase.h"
 #include "checknullpointer.h"
 #include <sstream>
@@ -226,29 +225,21 @@ void CheckStl::mismatchingContainersError(const Token *tok)
     reportError(tok, Severity::error, "mismatchingContainers", "Iterators of different containers are used together.");
 }
 
-void CheckStl::mismatchingContainers()
-{
-    static const char* const algorithm2_strings[] = { // func(begin1, end1
-        "adjacent_find", "all_of", "any_of", "binary_search", "copy", "copy_if", "count", "count_if", "equal", "equal_range",
-        "find", "find_if", "find_if_not", "for_each", "generate", "is_heap", "is_heap_until", "is_partitioned",
-        "is_permutation", "is_sorted", "is_sorted_until", "lower_bound", "make_heap", "max_element", "minmax_element",
-        "min_element", "mismatch", "move", "move_backward", "next_permutation", "none_of", "partition", "partition_copy",
-        "partition_point", "pop_heap", "prev_permutation", "push_heap", "random_shuffle", "remove", "remove_copy",
-        "remove_copy_if", "remove_if", "replace", "replace_copy", "replace_copy_if", "replace_if", "reverse", "reverse_copy",
-        "search_n", "shuffle", "sort", "sort_heap", "stable_partition", "stable_sort", "swap_ranges", "transform", "unique",
-        "unique_copy", "upper_bound"
-    };
-    static const char* const algorithm22_strings[] = { // func(begin1, end1, begin2, end2
-        "find_end", "find_first_of", "includes", "lexicographical_compare", "merge", "partial_sort_copy",
-        "search", "set_difference", "set_intersection", "set_symmetric_difference", "set_union"
-    };
-    static const char* const algorithm1x1_strings[] = { // func(begin1, x, end1
-        "inplace_merge", "nth_element", "partial_sort", "rotate", "rotate_copy"
-    };
-
-    static const std::set<std::string> algorithm2(algorithm2_strings, &algorithm2_strings[sizeof(algorithm2_strings) / sizeof(*algorithm2_strings)]);
-    static const std::set<std::string> algorithm22(algorithm22_strings, &algorithm22_strings[sizeof(algorithm22_strings) / sizeof(*algorithm22_strings)]);
-    static const std::set<std::string> algorithm1x1(algorithm1x1_strings, &algorithm1x1_strings[sizeof(algorithm1x1_strings) / sizeof(*algorithm1x1_strings)]);
+namespace {
+    const std::set<std::string> algorithm2 = make_container< std::set<std::string> >() // func(begin1, end1
+            << "adjacent_find" << "all_of" << "any_of" << "binary_search" << "copy" << "copy_if" << "count" << "count_if" << "equal" << "equal_range"
+            << "find" << "find_if" << "find_if_not" << "for_each" << "generate" << "is_heap" << "is_heap_until" << "is_partitioned"
+            << "is_permutation" << "is_sorted" << "is_sorted_until" << "lower_bound" << "make_heap" << "max_element" << "minmax_element"
+            << "min_element" << "mismatch" << "move" << "move_backward" << "next_permutation" << "none_of" << "partition" << "partition_copy"
+            << "partition_point" << "pop_heap" << "prev_permutation" << "push_heap" << "random_shuffle" << "remove" << "remove_copy"
+            << "remove_copy_if" << "remove_if" << "replace" << "replace_copy" << "replace_copy_if" << "replace_if" << "reverse" << "reverse_copy"
+            << "search_n" << "shuffle" << "sort" << "sort_heap" << "stable_partition" << "stable_sort" << "swap_ranges" << "transform" << "unique"
+            << "unique_copy" << "upper_bound" << "string" << "wstring" << "u16string" << "u32string";
+    const std::set<std::string> algorithm22 = make_container< std::set<std::string> >() // func(begin1 << end1 << begin2 << end2
+            << "find_end" << "find_first_of" << "includes" << "lexicographical_compare" << "merge" << "partial_sort_copy"
+            << "search" << "set_difference" << "set_intersection" << "set_symmetric_difference" << "set_union";
+    const std::set<std::string> algorithm1x1 = make_container< std::set<std::string> >()  // func(begin1 << x << end1
+            << "inplace_merge" << "nth_element" << "partial_sort" << "rotate" << "rotate_copy";
 
     static const std::string iteratorBeginFuncPattern = "begin|cbegin|rbegin|crbegin";
     static const std::string iteratorEndFuncPattern = "end|cend|rend|crend";
@@ -256,6 +247,10 @@ void CheckStl::mismatchingContainers()
     static const std::string pattern1x1_1 = "%name% . " + iteratorBeginFuncPattern + " ( ) , ";
     static const std::string pattern1x1_2 = "%name% . " + iteratorEndFuncPattern + " ( ) ,|)";
     static const std::string pattern2 = pattern1x1_1 + pattern1x1_2;
+}
+
+void CheckStl::mismatchingContainers()
+{
 
     // Check if different containers are used in various calls of standard functions
     const SymbolDatabase *symbolDatabase = _tokenizer->getSymbolDatabase();
@@ -293,6 +288,14 @@ void CheckStl::mismatchingContainers()
             tok = arg1->linkAt(-1);
         }
     }
+    for (unsigned int varid = 0; varid < symbolDatabase->getVariableListSize(); varid++) {
+        const Variable* var = symbolDatabase->getVariableFromVarId(varid);
+        if (var && var->isStlStringType() && Token::Match(var->nameToken(), "%var% (") && Token::Match(var->nameToken()->tokAt(2), pattern2.c_str())) {
+            if (var->nameToken()->strAt(2) != var->nameToken()->strAt(8)) {
+                mismatchingContainersError(var->nameToken());
+            }
+        }
+    }
 }
 
 
@@ -366,185 +369,64 @@ void CheckStl::stlOutOfBoundsError(const Token *tok, const std::string &num, con
         reportError(tok, Severity::error, "stlOutOfBounds", "When " + num + "==" + var + ".size(), " + var + "[" + num + "] is out of bounds.");
 }
 
-
-/**
- * @brief %Check for invalid iterator usage after erase/insert/etc
- */
-class EraseCheckLoop : public ExecutionPath {
-public:
-    static void checkScope(CheckStl *checkStl, const Token *it) {
-        const Token *tok = it;
-
-        // Search for the start of the loop body..
-        while (0 != (tok = tok->next())) {
-            if (tok->str() == "(")
-                tok = tok->link();
-            else if (tok->str() == ")")
-                break;
-
-            // reassigning iterator in loop head
-            else if (Token::Match(tok, "%name% =") && tok->str() == it->str())
-                break;
-        }
-
-        if (! Token::simpleMatch(tok, ") {"))
-            return;
-
-        EraseCheckLoop c(checkStl, it->varId(), it);
-        std::list<ExecutionPath *> checks;
-        checks.push_back(c.copy());
-        ExecutionPath::checkScope(tok->tokAt(2), checks);
-
-        c.end(checks, tok->link());
-
-        while (!checks.empty()) {
-            delete checks.back();
-            checks.pop_back();
-        }
-    }
-
-private:
-    /** Startup constructor */
-    EraseCheckLoop(Check *o, unsigned int varid, const Token* usetoken)
-        : ExecutionPath(o, varid), eraseToken(0), useToken(usetoken) {
-    }
-
-    /** @brief token where iterator is erased (non-zero => the iterator is invalid) */
-    const Token *eraseToken;
-
-    /** @brief name of the iterator */
-    const Token* useToken;
-
-    /** @brief Copy this check. Called from the ExecutionPath baseclass. */
-    ExecutionPath *copy() {
-        return new EraseCheckLoop(*this);
-    }
-
-    /** @brief is another execution path equal? */
-    bool is_equal(const ExecutionPath *e) const {
-        const EraseCheckLoop *c = static_cast<const EraseCheckLoop *>(e);
-        return (eraseToken == c->eraseToken);
-    }
-
-    /** @brief parse tokens */
-    const Token *parse(const Token &tok, std::list<ExecutionPath *> &checks) const {
-        // bail out if there are assignments. We don't check the assignments properly.
-        if (Token::Match(&tok, "[;{}] %var% =") || Token::Match(&tok, "= %var% ;")) {
-            ExecutionPath::bailOutVar(checks, tok.next()->varId());
-        }
-
-        // the loop stops here. Bail out all execution checks that reach
-        // this statement
-        if (Token::Match(&tok, "[;{}] break ;")) {
-            ExecutionPath::bailOut(checks);
-        }
-
-        // erasing iterator => it is invalidated
-        if (Token::Match(&tok, "erase ( ++|--| %name% )")) {
-            // check if there is a "it = ints.erase(it);" pattern. if so
-            // the it is not invalidated.
-            const Token *token = &tok;
-            while (nullptr != (token = token ? token->previous() : 0)) {
-                if (Token::Match(token, "[;{}]"))
-                    break;
-                else if (token->str() == "=")
-                    token = 0;
-            }
-
-            // the it is invalidated by the erase..
-            if (token) {
-                // get variable id for the iterator
-                unsigned int iteratorId = 0;
-                if (tok.tokAt(2)->isName())
-                    iteratorId = tok.tokAt(2)->varId();
-                else
-                    iteratorId = tok.tokAt(3)->varId();
-
-                // invalidate this iterator in the corresponding checks
-                for (std::list<ExecutionPath *>::const_iterator it = checks.begin(); it != checks.end(); ++it) {
-                    EraseCheckLoop *c = dynamic_cast<EraseCheckLoop *>(*it);
-                    if (c && c->varId == iteratorId) {
-                        c->eraseToken = &tok;
-                    }
-                }
-            }
-        }
-
-        // don't skip any tokens. return the token that we received.
-        return &tok;
-    }
-
-    /**
-     * Parse condition. @sa ExecutionPath::parseCondition
-     * @param tok first token in condition.
-     * @param checks The execution paths. All execution paths in the list are executed in the current scope
-     * @return true => bail out all checking
-     **/
-    bool parseCondition(const Token &tok, std::list<ExecutionPath *> &checks) {
-        // no checking of conditions.
-        (void)tok;
-        (void)checks;
-        return false;
-    }
-
-    /** @brief going out of scope - all execution paths end */
-    void end(const std::list<ExecutionPath *> &checks, const Token * /*tok*/) const {
-        // check if there are any invalid iterators. If so there is an error.
-        for (std::list<ExecutionPath *>::const_iterator it = checks.begin(); it != checks.end(); ++it) {
-            EraseCheckLoop *c = dynamic_cast<EraseCheckLoop *>(*it);
-            if (c && c->eraseToken) {
-                CheckStl *checkStl = dynamic_cast<CheckStl *>(c->owner);
-                if (checkStl) {
-                    checkStl->dereferenceErasedError(c->eraseToken, c->useToken, c->useToken->str());
-                }
-            }
-        }
-    }
-};
-
-
 void CheckStl::erase()
 {
     const SymbolDatabase* const symbolDatabase = _tokenizer->getSymbolDatabase();
 
     for (std::list<Scope>::const_iterator i = symbolDatabase->scopeList.begin(); i != symbolDatabase->scopeList.end(); ++i) {
-        const Token* const tok = i->classDef;
-        if (!tok)
-            continue;
-
-        if (i->type == Scope::eFor) {
-            for (const Token *tok2 = tok->tokAt(2); tok2; tok2 = tok2->next()) {
-                if (tok2->str() == ";") {
-                    if (Token::Match(tok2, "; %var% !=")) {
-                        // Get declaration token for var..
-                        const Variable *variableInfo = tok2->next()->variable();
-                        const Token *decltok = variableInfo ? variableInfo->typeEndToken() : nullptr;
-
-                        // Is variable an iterator?
-                        // If tok2->next() is an iterator, check scope
-                        if (decltok && Token::Match(decltok->tokAt(-2), "> :: iterator %varid%", tok2->next()->varId()))
-                            EraseCheckLoop::checkScope(this, tok2->next());
-                    }
-                    break;
-                }
-
-                if (Token::Match(tok2, "%name% = %name% . begin|rbegin|cbegin|crbegin ( ) ; %name% != %name% . end|rend|cend|crend ( )") &&
-                    tok2->str() == tok2->strAt(8) &&
-                    tok2->strAt(2) == tok2->strAt(10)) {
-                    EraseCheckLoop::checkScope(this, tok2);
-                    break;
-                }
-            }
-        }
-
-        else if (i->type == Scope::eWhile && Token::Match(tok, "while ( %var% !=")) {
-            const Variable* var = tok->tokAt(2)->variable();
-            if (var && Token::simpleMatch(var->typeEndToken()->tokAt(-2), "> :: iterator"))
-                EraseCheckLoop::checkScope(this, tok->tokAt(2));
+        if (i->type == Scope::eFor && Token::simpleMatch(i->classDef, "for (")) {
+            const Token *tok = i->classDef->linkAt(1);
+            if (!Token::Match(tok->tokAt(-3), "; ++| %var% ++| ) {"))
+                continue;
+            tok = tok->previous();
+            if (!tok->isName())
+                tok = tok->previous();
+            eraseCheckLoopVar(*i, tok->variable());
+        } else if (i->type == Scope::eWhile && Token::Match(i->classDef, "while ( %var% !=")) {
+            eraseCheckLoopVar(*i, i->classDef->tokAt(2)->variable());
         }
     }
 }
 
+void CheckStl::eraseCheckLoopVar(const Scope &scope, const Variable *var)
+{
+    if (!var || !Token::simpleMatch(var->typeEndToken(), "iterator"))
+        return;
+    for (const Token *tok = scope.classStart; tok != scope.classEnd; tok = tok->next()) {
+        if (tok->str() != "(")
+            continue;
+        if (!Token::Match(tok->tokAt(-2), ". erase ( ++| %varid% )", var->declarationId()))
+            continue;
+        if (Token::simpleMatch(tok->astParent(), "="))
+            continue;
+        // Iterator is invalid..
+        unsigned int indentlevel = 0U;
+        const Token *tok2 = tok->link();
+        for (; tok2 != scope.classEnd; tok2 = tok2->next()) {
+            if (tok2->str() == "{") {
+                ++indentlevel;
+                continue;
+            }
+            if (tok2->str() == "}") {
+                if (indentlevel > 0U)
+                    --indentlevel;
+                else if (Token::simpleMatch(tok2, "} else {"))
+                    tok2 = tok2->linkAt(2);
+                continue;
+            }
+            if (tok2->varId() == var->declarationId()) {
+                if (Token::simpleMatch(tok2->next(), "="))
+                    break;
+                dereferenceErasedError(tok, tok2, tok2->str());
+                break;
+            }
+            if (indentlevel == 0U && Token::Match(tok2, "break|return|goto"))
+                break;
+        }
+        if (tok2 == scope.classEnd)
+            dereferenceErasedError(tok, scope.classDef, var->nameToken()->str());
+    }
+}
 
 void CheckStl::pushback()
 {
@@ -746,7 +628,7 @@ static bool if_findCompare(const Token * const tokBack)
     if (!tok)
         return true;
     if (tok->isComparisonOp())
-        return true;
+        return (!tok->astOperand1()->isNumber() && !tok->astOperand2()->isNumber());
     if (tok->isArithmeticalOp()) // result is used in some calculation
         return true;  // TODO: check if there is a comparison of the result somewhere
     if (tok->str() == ".")
@@ -769,11 +651,7 @@ void CheckStl::if_find()
         if ((i->type != Scope::eIf && i->type != Scope::eWhile) || !i->classDef)
             continue;
 
-        const Token* tok = i->classDef->next();
-        if (tok->str() == "if")
-            tok = tok->next();
-
-        for (const Token* const end = tok->link(); tok != end; tok = (tok == end) ? end : tok->next()) {
+        for (const Token *tok = i->classDef; tok->str() != "{"; tok = tok->next()) {
             const Token* funcTok = nullptr;
             const Library::Container* container = nullptr;
 
@@ -843,21 +721,12 @@ void CheckStl::if_findError(const Token *tok, bool str)
 /**
  * Is container.size() slow?
  */
-static bool isContainerSizeSlow(const Token *tok)
+static bool isCpp03ContainerSizeSlow(const Token *tok)
 {
-    // THIS ARRAY MUST BE ORDERED ALPHABETICALLY
-    static const char* stl_size_slow[] = {
-        "array", "bitset",
-        "forward_list", "hash_map", "hash_multimap", "hash_set",
-        "list", "map", "multimap", "multiset",
-        "priority_queue", "queue", "set", "stack", "unordered_map",
-        "unordered_multimap", "unordered_multiset", "unordered_set"
-    };
-
     if (!tok)
         return false;
     const Variable* var = tok->variable();
-    return var && var->isStlType(stl_size_slow);
+    return var && var->isStlType("list");
 }
 
 void CheckStl::size()
@@ -865,6 +734,9 @@ void CheckStl::size()
     if (!_settings->isEnabled("performance"))
         return;
 
+    if (_settings->standards.cpp == Standards::CPP11)
+        return;
+
     const SymbolDatabase* const symbolDatabase = _tokenizer->getSymbolDatabase();
     const std::size_t functions = symbolDatabase->functionScopes.size();
     for (std::size_t i = 0; i < functions; ++i) {
@@ -883,21 +755,21 @@ void CheckStl::size()
                 // check for comparison to zero
                 if ((tok->previous() && !tok->previous()->isArithmeticalOp() && Token::Match(end, "==|<=|!=|> 0")) ||
                     (end->next() && !end->next()->isArithmeticalOp() && Token::Match(tok->tokAt(-2), "0 ==|>=|!=|<"))) {
-                    if (isContainerSizeSlow(tok1))
+                    if (isCpp03ContainerSizeSlow(tok1))
                         sizeError(tok1);
                 }
 
                 // check for comparison to one
                 if ((tok->previous() && !tok->previous()->isArithmeticalOp() && Token::Match(end, ">=|< 1") && !end->tokAt(2)->isArithmeticalOp()) ||
                     (end->next() && !end->next()->isArithmeticalOp() && Token::Match(tok->tokAt(-2), "1 <=|>") && !tok->tokAt(-3)->isArithmeticalOp())) {
-                    if (isContainerSizeSlow(tok1))
+                    if (isCpp03ContainerSizeSlow(tok1))
                         sizeError(tok1);
                 }
 
                 // check for using as boolean expression
                 else if ((Token::Match(tok->tokAt(-2), "if|while (") && end->str() == ")") ||
-                         (tok->previous()->type() == Token::eLogicalOp && Token::Match(end, "&&|)|,|;|%oror%"))) {
-                    if (isContainerSizeSlow(tok1))
+                         (tok->previous()->tokType() == Token::eLogicalOp && Token::Match(end, "&&|)|,|;|%oror%"))) {
+                    if (isCpp03ContainerSizeSlow(tok1))
                         sizeError(tok1);
                 }
             }
@@ -1042,18 +914,17 @@ static bool isLocal(const Token *tok)
     return var && !var->isStatic() && var->isLocal();
 }
 
+namespace {
+    static const std::set<std::string> stl_string = make_container< std::set<std::string> >() <<
+            "string" <<  "u16string" <<  "u32string" <<  "wstring" ;
+    static const std::set<std::string> stl_string_stream = make_container< std::set<std::string> >() <<
+            "istringstream" <<  "ostringstream" <<  "stringstream" <<  "wstringstream" ;
+}
+
 void CheckStl::string_c_str()
 {
     const bool printInconclusive = _settings->inconclusive;
     const bool printPerformance = _settings->isEnabled("performance");
-    // THIS ARRAY MUST BE ORDERED ALPHABETICALLY
-    static const char* const stl_string[] = {
-        "string", "u16string", "u32string", "wstring"
-    };
-    // THIS ARRAY MUST BE ORDERED ALPHABETICALLY
-    static const char* const stl_string_stream[] = {
-        "istringstream", "ostringstream", "stringstream", "wstringstream"
-    };
 
     const SymbolDatabase* symbolDatabase = _tokenizer->getSymbolDatabase();
 
@@ -1350,6 +1221,15 @@ void CheckStl::autoPointerMallocError(const Token *tok, const std::string& alloc
     reportError(tok, Severity::error, "useAutoPointerMalloc", summary + "\n" + verbose);
 }
 
+namespace {
+    static const std::set<std::string> stl_containers_with_empty_and_clear = make_container< std::set<std::string> >() <<
+            "deque" <<  "forward_list" <<  "list" <<
+            "map" <<  "multimap" <<  "multiset" <<  "set" <<  "string" <<
+            "unordered_map" <<  "unordered_multimap" <<  "unordered_multiset" <<
+            "unordered_set" <<  "vector" <<  "wstring";
+
+}
+
 void CheckStl::uselessCalls()
 {
     const bool printPerformance = _settings->isEnabled("performance");
@@ -1357,17 +1237,6 @@ void CheckStl::uselessCalls()
     if (!printPerformance && !printWarning)
         return;
 
-    // THIS ARRAY MUST BE ORDERED ALPHABETICALLY
-    static const char* const stl_string[] = {
-        "string", "u16string", "u32string", "wstring"
-    };
-    // THIS ARRAY MUST BE ORDERED ALPHABETICALLY
-    static const char* const stl_containers_with_empty_and_clear[] = {
-        "deque", "forward_list", "list",
-        "map", "multimap", "multiset", "set", "string",
-        "unordered_map", "unordered_multimap", "unordered_multiset",
-        "unordered_set", "vector", "wstring"
-    };
 
     const SymbolDatabase* symbolDatabase = _tokenizer->getSymbolDatabase();
     const std::size_t functions = symbolDatabase->functionScopes.size();
@@ -1514,6 +1383,43 @@ void CheckStl::dereferenceInvalidIteratorError(const Token* deref, const std::st
 
 
 
+void CheckStl::readingEmptyStlContainer_parseUsage(const Token* tok, bool map, std::set<unsigned int>& empty, bool noerror)
+{
+    // Check for various conditions for the way stl containers and variables can be used
+    if (tok->strAt(1) == "=" || (tok->strAt(1) == "[" && Token::simpleMatch(tok->linkAt(1), "] ="))) {
+        // Assignment (LHS)
+        empty.erase(tok->varId());
+    } else if (Token::Match(tok, "%name% [")) {
+        // Access through operator[]
+        if (map) { // operator[] inserts an element, if used on a std::map
+            if (!noerror && tok->strAt(-1) == "=")
+                readingEmptyStlContainerError(tok);
+            empty.erase(tok->varId());
+        } else if (!noerror)
+            readingEmptyStlContainerError(tok);
+    } else if (Token::Match(tok, "%name% . %type% (")) {
+        // Member function call
+        if (Token::Match(tok->tokAt(2), "find|at|data|c_str|back|front|empty|top|size|count")) { // These functions read from the container
+            if (!noerror)
+                readingEmptyStlContainerError(tok);
+        } else
+            empty.erase(tok->varId());
+    } else if (tok->strAt(-1) == "=") {
+        // Assignment (RHS)
+        if (!noerror)
+            readingEmptyStlContainerError(tok);
+    } else {
+        // Unknown usage. Assume it is initialized.
+        empty.erase(tok->varId());
+    }
+}
+
+namespace {
+    static const std::set<std::string> MAP_STL_CONTAINERS = make_container< std::set<std::string> >() <<
+            "map" << "multimap" << "unordered_map" << "unordered_multimap" ;
+    static const std::set<std::string> NONMAP_STL_CONTAINERS = make_container< std::set<std::string> >() <<
+            "deque" << "forward_list" << "list" << "multiset" << "queue" << "set" << "stack" << "string" << "unordered_multiset" << "unordered_set" << "vector";
+}
 
 void CheckStl::readingEmptyStlContainer()
 {
@@ -1526,23 +1432,34 @@ void CheckStl::readingEmptyStlContainer()
     std::set<unsigned int> empty_map; // empty std::map-like instances of STL containers
     std::set<unsigned int> empty_nonmap; // empty non-std::map-like instances of STL containers
 
-    static const char *MAP_STL_CONTAINERS[] = { "map", "multimap", "unordered_map", "unordered_multimap" };
-    static const char *NONMAP_STL_CONTAINERS[] = { "deque", "forward_list", "list", "multiset", "queue", "set", "stack", "string", "unordered_multiset", "unordered_set", "vector" };
-
     const std::list<Scope>& scopeList = _tokenizer->getSymbolDatabase()->scopeList;
 
     for (std::list<Scope>::const_iterator i = scopeList.begin(); i != scopeList.end(); ++i) {
         if (i->type != Scope::eFunction)
             continue;
 
-        const Token* restartTok = nullptr;
         for (const Token *tok = i->classStart->next(); tok != i->classEnd; tok = tok->next()) {
             if (Token::Match(tok, "for|while")) { // Loops and end of scope clear the sets.
-                restartTok = tok->linkAt(1); // Check condition to catch looping over empty containers
-            } else if (tok == restartTok || Token::Match(tok, "do|}")) {
+                const Token* tok2 = tok->linkAt(1);
+                if (!tok2)
+                    continue;
+                tok2 = tok2->next();
+                for (const Token* end2 = tok2->link(); tok2 && tok2 != end2; tok2 = tok2->next()) {
+                    if (!tok2->varId())
+                        continue;
+
+                    const bool map = empty_map.find(tok2->varId()) != empty_map.end();
+                    if (!map && empty_nonmap.find(tok2->varId()) == empty_nonmap.end())
+                        continue;
+
+                    if (map)
+                        readingEmptyStlContainer_parseUsage(tok2, true, empty_map, true);
+                    else
+                        readingEmptyStlContainer_parseUsage(tok2, false, empty_nonmap, true);
+                }
+            } else if (Token::Match(tok, "do|}|break|case")) {
                 empty_map.clear();
                 empty_nonmap.clear();
-                restartTok = nullptr;
             }
 
             if (!tok->varId())
@@ -1571,39 +1488,10 @@ void CheckStl::readingEmptyStlContainer()
             if (!map && empty_nonmap.find(tok->varId()) == empty_nonmap.end())
                 continue;
 
-            // Check for various conditions for the way stl containers and variables can be used
-            if (tok->strAt(1) == "=" || (tok->strAt(1) == "[" && Token::simpleMatch(tok->linkAt(1), "] ="))) {
-                // Assignment (LHS)
-                if (map)
-                    empty_map.erase(tok->varId());
-                else
-                    empty_nonmap.erase(tok->varId());
-            } else if (Token::Match(tok, "%name% [")) {
-                // Access through operator[]
-                if (map) { // operator[] inserts an element, if used on a std::map
-                    if (tok->strAt(-1) == "=")
-                        readingEmptyStlContainerError(tok);
-                    empty_map.erase(tok->varId());
-                } else
-                    readingEmptyStlContainerError(tok);
-            } else if (Token::Match(tok, "%name% . %type% (")) {
-                // Member function call
-                if (Token::Match(tok->tokAt(2), "find|at|data|c_str|back|front|empty|top|size|count")) // These functions read from the container
-                    readingEmptyStlContainerError(tok);
-                else if (map)
-                    empty_map.erase(tok->varId());
-                else
-                    empty_nonmap.erase(tok->varId());
-            } else if (tok->strAt(-1) == "=") {
-                // Assignment (RHS)
-                readingEmptyStlContainerError(tok);
-            } else {
-                // Unknown usage. Assume it is initialized.
-                if (map)
-                    empty_map.erase(tok->varId());
-                else
-                    empty_nonmap.erase(tok->varId());
-            }
+            if (map)
+                readingEmptyStlContainer_parseUsage(tok, true, empty_map, false);
+            else
+                readingEmptyStlContainer_parseUsage(tok, false, empty_nonmap, false);
         }
         empty_map.clear();
         empty_nonmap.clear();
@@ -1612,5 +1500,5 @@ void CheckStl::readingEmptyStlContainer()
 
 void CheckStl::readingEmptyStlContainerError(const Token *tok)
 {
-    reportError(tok, Severity::style, "reademptycontainer", "Reading from empty STL container", 0U, true);
+    reportError(tok, Severity::style, "reademptycontainer", "Reading from empty STL container '" + (tok ? tok->str() : std::string("var")) + "'", 0U, true);
 }
