@@ -59,8 +59,12 @@ public:
         checkString.checkIncorrectStringCompare();
         checkString.checkAlwaysTrueOrFalseStringCompare();
         checkString.sprintfOverlappingData();
+        checkString.stringLiteralWrite();
     }
 
+    /** @brief undefined behaviour, writing string literal */
+    void stringLiteralWrite();
+
     /** @brief str plus char (unusual pointer arithmetic) */
     void strPlusChar();
 
@@ -77,6 +81,7 @@ public:
     void sprintfOverlappingData();
 
 private:
+    void stringLiteralWriteError(const Token *tok, const Token *strValue);
     void sprintfOverlappingDataError(const Token *tok, const std::string &varname);
     void strPlusCharError(const Token *tok);
     void incorrectStringCompareError(const Token *tok, const std::string& func, const std::string &string);
@@ -89,6 +94,7 @@ private:
     void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings) const {
         CheckString c(0, settings, errorLogger);
 
+        c.stringLiteralWriteError(0,0);
         c.sprintfOverlappingDataError(0, "varname");
         c.strPlusCharError(0);
         c.incorrectStringCompareError(0, "substr", "\"Hello World\"");
