@@ -23,6 +23,7 @@
 #include "mathlib.h"
 #include "token.h"
 #include "symboldatabase.h"
+#include "astutils.h"
 
 #include <string>
 #include <algorithm>
@@ -35,8 +36,10 @@ Library::Error Library::load(const char exename[], const char path[])
 {
     if (std::strchr(path,',') != nullptr) {
         std::string p(path);
-        while (p.find(",") != std::string::npos) {
-            const std::string::size_type pos = p.find(",");
+        for (;;) {
+            const std::string::size_type pos = p.find(',');
+            if (pos == std::string::npos)
+                break;
             const Error &e = load(exename, p.substr(0,pos).c_str());
             if (e.errorcode != OK)
                 return e;
@@ -110,11 +113,11 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
         return Error(UNSUPPORTED_FORMAT, rootnode->Name());
 
     const char* format_string = rootnode->Attribute("format");
-    int format = 1;
+    int format = 1; // Assume format version 1 if nothing else is specified (very old .cfg files had no 'format' attribute)
     if (format_string)
         format = atoi(format_string);
 
-    if (format > 1)
+    if (format > 2 || format <= 0)
         return Error(UNSUPPORTED_FORMAT);
 
     std::set<std::string> unknown_elements;
@@ -178,119 +181,19 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
             if (name_char == nullptr)
                 return Error(MISSING_ATTRIBUTE, "name");
             std::string name = name_char;
-
-            for (const tinyxml2::XMLElement *functionnode = node->FirstChildElement(); functionnode; functionnode = functionnode->NextSiblingElement()) {
-                const std::string functionnodename = functionnode->Name();
-                if (functionnodename == "noreturn")
-                    _noreturn[name] = (strcmp(functionnode->GetText(), "true") == 0);
-                else if (functionnodename == "pure")
-                    functionpure.insert(name);
-                else if (functionnodename == "const") {
-                    functionconst.insert(name);
-                    functionpure.insert(name); // a constant function is pure
-                } else if (functionnodename == "leak-ignore")
-                    leakignore.insert(name);
-                else if (functionnodename == "use-retval")
-                    useretval.insert(name);
-                else if (functionnodename == "arg" && functionnode->Attribute("nr") != nullptr) {
-                    const bool bAnyArg = strcmp(functionnode->Attribute("nr"),"any")==0;
-                    const int nr = (bAnyArg) ? -1 : atoi(functionnode->Attribute("nr"));
-                    bool notbool = false;
-                    bool notnull = false;
-                    bool notuninit = false;
-                    bool formatstr = false;
-                    bool strz = false;
-                    std::string& valid = argumentChecks[name][nr].valid;
-                    std::list<ArgumentChecks::MinSize>& minsizes = argumentChecks[name][nr].minsizes;
-                    for (const tinyxml2::XMLElement *argnode = functionnode->FirstChildElement(); argnode; argnode = argnode->NextSiblingElement()) {
-                        const std::string argnodename = argnode->Name();
-                        if (argnodename == "not-bool")
-                            notbool = true;
-                        else if (argnodename == "not-null")
-                            notnull = true;
-                        else if (argnodename == "not-uninit")
-                            notuninit = true;
-                        else if (argnodename == "formatstr")
-                            formatstr = true;
-                        else if (argnodename == "strz")
-                            strz = true;
-                        else if (argnodename == "valid") {
-                            // Validate the validation expression
-                            const char *p = argnode->GetText();
-                            bool error = false;
-                            bool range = false;
-                            for (; *p; p++) {
-                                if (std::isdigit(*p))
-                                    error |= (*(p+1) == '-');
-                                else if (*p == ':')
-                                    error |= range;
-                                else if (*p == '-')
-                                    error |= (!std::isdigit(*(p+1)));
-                                else if (*p == ',')
-                                    range = false;
-                                else
-                                    error = true;
-
-                                range |= (*p == ':');
-                            }
-                            if (error)
-                                return Error(BAD_ATTRIBUTE_VALUE, argnode->GetText());
-
-                            // Set validation expression
-                            valid = argnode->GetText();
-                        }
-
-                        else if (argnodename == "minsize") {
-                            const char *typeattr = argnode->Attribute("type");
-                            if (!typeattr)
-                                return Error(MISSING_ATTRIBUTE, "type");
-
-                            ArgumentChecks::MinSize::Type type;
-                            if (strcmp(typeattr,"strlen")==0)
-                                type = ArgumentChecks::MinSize::STRLEN;
-                            else if (strcmp(typeattr,"argvalue")==0)
-                                type = ArgumentChecks::MinSize::ARGVALUE;
-                            else if (strcmp(typeattr,"sizeof")==0)
-                                type = ArgumentChecks::MinSize::SIZEOF;
-                            else if (strcmp(typeattr,"mul")==0)
-                                type = ArgumentChecks::MinSize::MUL;
-                            else
-                                return Error(BAD_ATTRIBUTE_VALUE, typeattr);
-
-                            const char *argattr  = argnode->Attribute("arg");
-                            if (!argattr)
-                                return Error(MISSING_ATTRIBUTE, "arg");
-                            if (strlen(argattr) != 1 || argattr[0]<'0' || argattr[0]>'9')
-                                return Error(BAD_ATTRIBUTE_VALUE, argattr);
-
-                            minsizes.push_back(ArgumentChecks::MinSize(type,argattr[0]-'0'));
-                            if (type == ArgumentChecks::MinSize::MUL) {
-                                const char *arg2attr  = argnode->Attribute("arg2");
-                                if (!arg2attr)
-                                    return Error(MISSING_ATTRIBUTE, "arg2");
-                                if (strlen(arg2attr) != 1 || arg2attr[0]<'0' || arg2attr[0]>'9')
-                                    return Error(BAD_ATTRIBUTE_VALUE, arg2attr);
-                                minsizes.back().arg2 = arg2attr[0] - '0';
-                            }
-                        }
-
-                        else
-                            unknown_elements.insert(argnodename);
-                    }
-                    argumentChecks[name][nr].notbool   = notbool;
-                    argumentChecks[name][nr].notnull   = notnull;
-                    argumentChecks[name][nr].notuninit = notuninit;
-                    argumentChecks[name][nr].formatstr = formatstr;
-                    argumentChecks[name][nr].strz      = strz;
-                } else if (functionnodename == "ignorefunction") {
-                    _ignorefunction.insert(name);
-                } else if (functionnodename == "formatstr") {
-                    const tinyxml2::XMLAttribute* scan = functionnode->FindAttribute("scan");
-                    const tinyxml2::XMLAttribute* secure = functionnode->FindAttribute("secure");
-                    _formatstr[name] = std::make_pair(scan && scan->BoolValue(), secure && secure->BoolValue());
-                } else
-                    unknown_elements.insert(functionnodename);
+            for (;;) {
+                const std::string::size_type pos = name.find(',');
+                if (pos == std::string::npos)
+                    break;
+                const Error &err = loadFunction(node, name.substr(0,pos), unknown_elements);
+                if (err.errorcode != ErrorCode::OK)
+                    return err;
+                name.erase(0,pos+1);
             }
+
+            const Error &err = loadFunction(node, name, unknown_elements);
+            if (err.errorcode != ErrorCode::OK)
+                return err;
         }
 
         else if (nodename == "reflection") {
@@ -581,6 +484,126 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
     return Error(OK);
 }
 
+Library::Error Library::loadFunction(const tinyxml2::XMLElement * const node, const std::string &name, std::set<std::string> &unknown_elements)
+{
+    if (name.empty())
+        return Error(OK);
+
+    for (const tinyxml2::XMLElement *functionnode = node->FirstChildElement(); functionnode; functionnode = functionnode->NextSiblingElement()) {
+        const std::string functionnodename = functionnode->Name();
+        if (functionnodename == "noreturn")
+            _noreturn[name] = (strcmp(functionnode->GetText(), "true") == 0);
+        else if (functionnodename == "pure")
+            functionpure.insert(name);
+        else if (functionnodename == "const") {
+            functionconst.insert(name);
+            functionpure.insert(name); // a constant function is pure
+        } else if (functionnodename == "leak-ignore")
+            leakignore.insert(name);
+        else if (functionnodename == "use-retval")
+            _useretval.insert(name);
+        else if (functionnodename == "arg" && functionnode->Attribute("nr") != nullptr) {
+            const bool bAnyArg = strcmp(functionnode->Attribute("nr"),"any")==0;
+            const int nr = (bAnyArg) ? -1 : atoi(functionnode->Attribute("nr"));
+            bool notbool = false;
+            bool notnull = false;
+            bool notuninit = false;
+            bool formatstr = false;
+            bool strz = false;
+            std::string& valid = argumentChecks[name][nr].valid;
+            std::list<ArgumentChecks::MinSize>& minsizes = argumentChecks[name][nr].minsizes;
+            for (const tinyxml2::XMLElement *argnode = functionnode->FirstChildElement(); argnode; argnode = argnode->NextSiblingElement()) {
+                const std::string argnodename = argnode->Name();
+                if (argnodename == "not-bool")
+                    notbool = true;
+                else if (argnodename == "not-null")
+                    notnull = true;
+                else if (argnodename == "not-uninit")
+                    notuninit = true;
+                else if (argnodename == "formatstr")
+                    formatstr = true;
+                else if (argnodename == "strz")
+                    strz = true;
+                else if (argnodename == "valid") {
+                    // Validate the validation expression
+                    const char *p = argnode->GetText();
+                    bool error = false;
+                    bool range = false;
+                    for (; *p; p++) {
+                        if (std::isdigit(*p))
+                            error |= (*(p+1) == '-');
+                        else if (*p == ':')
+                            error |= range;
+                        else if (*p == '-')
+                            error |= (!std::isdigit(*(p+1)));
+                        else if (*p == ',')
+                            range = false;
+                        else
+                            error = true;
+
+                        range |= (*p == ':');
+                    }
+                    if (error)
+                        return Error(BAD_ATTRIBUTE_VALUE, argnode->GetText());
+
+                    // Set validation expression
+                    valid = argnode->GetText();
+                }
+
+                else if (argnodename == "minsize") {
+                    const char *typeattr = argnode->Attribute("type");
+                    if (!typeattr)
+                        return Error(MISSING_ATTRIBUTE, "type");
+
+                    ArgumentChecks::MinSize::Type type;
+                    if (strcmp(typeattr,"strlen")==0)
+                        type = ArgumentChecks::MinSize::STRLEN;
+                    else if (strcmp(typeattr,"argvalue")==0)
+                        type = ArgumentChecks::MinSize::ARGVALUE;
+                    else if (strcmp(typeattr,"sizeof")==0)
+                        type = ArgumentChecks::MinSize::SIZEOF;
+                    else if (strcmp(typeattr,"mul")==0)
+                        type = ArgumentChecks::MinSize::MUL;
+                    else
+                        return Error(BAD_ATTRIBUTE_VALUE, typeattr);
+
+                    const char *argattr  = argnode->Attribute("arg");
+                    if (!argattr)
+                        return Error(MISSING_ATTRIBUTE, "arg");
+                    if (strlen(argattr) != 1 || argattr[0]<'0' || argattr[0]>'9')
+                        return Error(BAD_ATTRIBUTE_VALUE, argattr);
+
+                    minsizes.push_back(ArgumentChecks::MinSize(type,argattr[0]-'0'));
+                    if (type == ArgumentChecks::MinSize::MUL) {
+                        const char *arg2attr  = argnode->Attribute("arg2");
+                        if (!arg2attr)
+                            return Error(MISSING_ATTRIBUTE, "arg2");
+                        if (strlen(arg2attr) != 1 || arg2attr[0]<'0' || arg2attr[0]>'9')
+                            return Error(BAD_ATTRIBUTE_VALUE, arg2attr);
+                        minsizes.back().arg2 = arg2attr[0] - '0';
+                    }
+                }
+
+                else
+                    unknown_elements.insert(argnodename);
+            }
+            argumentChecks[name][nr].notbool   = notbool;
+            argumentChecks[name][nr].notnull   = notnull;
+            argumentChecks[name][nr].notuninit = notuninit;
+            argumentChecks[name][nr].formatstr = formatstr;
+            argumentChecks[name][nr].strz      = strz;
+        } else if (functionnodename == "ignorefunction") {
+            _ignorefunction.insert(name);
+        } else if (functionnodename == "formatstr") {
+            const tinyxml2::XMLAttribute* scan = functionnode->FindAttribute("scan");
+            const tinyxml2::XMLAttribute* secure = functionnode->FindAttribute("secure");
+            _formatstr[name] = std::make_pair(scan && scan->BoolValue(), secure && secure->BoolValue());
+        } else
+            unknown_elements.insert(functionnodename);
+    }
+    return Error(OK);
+}
+
 bool Library::isargvalid(const Token *ftok, int argnr, const MathLib::bigint argvalue) const
 {
     const ArgumentChecks *ac = getarg(ftok, argnr);
@@ -608,12 +631,79 @@ bool Library::isargvalid(const Token *ftok, int argnr, const MathLib::bigint arg
     return false;
 }
 
+static std::string functionName(const Token *ftok, bool *error)
+{
+    if (!ftok) {
+        *error = true;
+        return "";
+    }
+    if (ftok->isName())
+        return ftok->str();
+    if (ftok->str() == "::") {
+        if (!ftok->astOperand2())
+            return functionName(ftok->astOperand1(), error);
+        return functionName(ftok->astOperand1(),error) + "::" + functionName(ftok->astOperand2(),error);
+    }
+    if (ftok->str() == "." && ftok->astOperand1()) {
+        const std::string type = astCanonicalType(ftok->astOperand1());
+        if (type.empty()) {
+            *error = true;
+            return "";
+        }
+
+        return type + "::" + functionName(ftok->astOperand2(),error);
+    }
+    *error = true;
+    return "";
+}
+
+static std::string functionName(const Token *ftok)
+{
+    if (!Token::Match(ftok, "%name% ("))
+        return "";
+
+    // Lookup function name using AST..
+    if (ftok->astParent()) {
+        bool error = false;
+        std::string ret = functionName(ftok->next()->astOperand1(), &error);
+        return error ? std::string() : ret;
+    }
+
+    // Lookup function name without using AST..
+    if (Token::simpleMatch(ftok->previous(), "."))
+        return "";
+    if (!Token::Match(ftok->tokAt(-2), "%name% ::"))
+        return ftok->str();
+    std::string ret(ftok->str());
+    ftok = ftok->tokAt(-2);
+    while (Token::Match(ftok, "%name% ::")) {
+        ret = ftok->str() + "::" + ret;
+        ftok = ftok->tokAt(-2);
+    }
+    return ret;
+}
+
+/** get allocation id for function */
+int Library::alloc(const Token *tok) const
+{
+    const std::string funcname = functionName(tok);
+    return isNotLibraryFunction(tok) && argumentChecks.find(funcname) != argumentChecks.end() ? 0 : getid(_alloc, funcname);
+}
+
+/** get deallocation id for function */
+int Library::dealloc(const Token *tok) const
+{
+    const std::string funcname = functionName(tok);
+    return isNotLibraryFunction(tok) && argumentChecks.find(funcname) != argumentChecks.end() ? 0 : getid(_dealloc, funcname);
+}
+
+
 const Library::ArgumentChecks * Library::getarg(const Token *ftok, int argnr) const
 {
     if (isNotLibraryFunction(ftok))
         return nullptr;
     std::map<std::string, std::map<int, ArgumentChecks> >::const_iterator it1;
-    it1 = argumentChecks.find(ftok->str());
+    it1 = argumentChecks.find(functionName(ftok));
     if (it1 == argumentChecks.end())
         return nullptr;
     const std::map<int,ArgumentChecks>::const_iterator it2 = it1->second.find(argnr);
@@ -678,13 +768,10 @@ const Library::Container* Library::detectContainer(const Token* typeStart) const
     }
     return nullptr;
 }
+
 // returns true if ftok is not a library function
 bool Library::isNotLibraryFunction(const Token *ftok) const
 {
-    // methods are not library functions
-    // called from tokenizer, ast is not created properly yet
-    if (Token::Match(ftok->previous(),"::|."))
-        return true;
     if (ftok->function() && ftok->function()->nestedIn && ftok->function()->nestedIn->type != Scope::eGlobal)
         return true;
 
@@ -701,7 +788,7 @@ bool Library::isNotLibraryFunction(const Token *ftok) const
         else if (tok->link() && Token::Match(tok, "<|(|["))
             tok = tok->link();
     }
-    const std::map<std::string, std::map<int, ArgumentChecks> >::const_iterator it = argumentChecks.find(ftok->str());
+    const std::map<std::string, std::map<int, ArgumentChecks> >::const_iterator it = argumentChecks.find(functionName(ftok));
     if (it == argumentChecks.end())
         return (callargs != 0);
     int args = 0;
@@ -714,13 +801,19 @@ bool Library::isNotLibraryFunction(const Token *ftok) const
     return args != callargs;
 }
 
+bool Library::isUseRetVal(const Token* ftok) const
+{
+    return (!isNotLibraryFunction(ftok) &&
+            _useretval.find(functionName(ftok)) != _useretval.end());
+}
+
 bool Library::isnoreturn(const Token *ftok) const
 {
     if (ftok->function() && ftok->function()->isAttributeNoreturn())
         return true;
     if (isNotLibraryFunction(ftok))
         return false;
-    std::map<std::string, bool>::const_iterator it = _noreturn.find(ftok->str());
+    std::map<std::string, bool>::const_iterator it = _noreturn.find(functionName(ftok));
     return (it != _noreturn.end() && it->second);
 }
 
@@ -730,7 +823,7 @@ bool Library::isnotnoreturn(const Token *ftok) const
         return false;
     if (isNotLibraryFunction(ftok))
         return false;
-    std::map<std::string, bool>::const_iterator it = _noreturn.find(ftok->str());
+    std::map<std::string, bool>::const_iterator it = _noreturn.find(functionName(ftok));
     return (it != _noreturn.end() && !it->second);
 }
 
