@@ -31,6 +31,7 @@
 #include "config.h"
 #include "token.h"
 #include "mathlib.h"
+#include "utils.h"
 
 class Tokenizer;
 class Settings;
@@ -118,10 +119,17 @@ public:
 
     /**
     * Check for circulare dependencies, i.e. loops within the class hierarchie
-    * @param anchestors list of anchestors. For internal usage only, clients should not supply this argument.
+    * @param ancestors list of ancestors. For internal usage only, clients should not supply this argument.
     * @return true if there is a circular dependency
     */
-    bool hasCircularDependencies(std::set<BaseInfo>* anchestors = 0) const;
+    bool hasCircularDependencies(std::set<BaseInfo>* ancestors = nullptr) const;
+
+    /**
+    * Check for dependency
+    * @param ancestor potential ancestor
+    * @return true if there is a dependency
+    */
+    bool findDependency(const Type* ancestor) const;
 };
 
 /** @brief Information about a member variable. */
@@ -366,7 +374,7 @@ public:
      * @return true if array, false if not
      */
     bool isArray() const {
-        return getFlag(fIsArray);
+        return getFlag(fIsArray) && !getFlag(fIsPointer);
     }
 
     /**
@@ -377,6 +385,20 @@ public:
         return getFlag(fIsPointer);
     }
 
+    /**
+     * Is variable a pointer to an array
+     * @return true if pointer to array, false otherwise
+     */
+    bool isPointerToArray() const {
+        return isPointer() && getFlag(fIsArray);
+    }
+
+    /**
+     * Is variable an array of pointers
+     * @return true if array or pointers, false otherwise
+     */
+    bool isPointerArray() const;
+
     /**
      * Is array or pointer variable.
      * @return true if pointer or array, false otherwise
@@ -488,12 +510,25 @@ public:
      *   ...
      *   const char *str[] = {"string", "wstring"};
      *   sVar->isStlType(str) == true
-     * @param stlTypes array of stl types in alphabetical order
+     * @param stlType stl type
      * @return true if it is an stl type and its type matches any of the types in 'stlTypes'
      */
-    template <std::size_t array_length>
-    bool isStlType(const char* const(&stlTypes)[array_length]) const {
-        return isStlType() && std::binary_search(stlTypes, stlTypes + array_length, _start->strAt(2));
+    bool isStlType(const std::string& stlType) const {
+        return isStlType() && stlType==_start->strAt(2);
+    }
+
+    /**
+     * Checks if the variable is of any of the STL types passed as arguments ('std::')
+     * E.g.:
+     *   std::string s;
+     *   ...
+     *   const std::set<std::string> str = make_container< std::set<std::string> >() << "string" << "wstring";
+     *   sVar->isStlType(str) == true
+     * @param stlTypes set of stl types
+     * @return true if it is an stl type and its type matches any of the types in 'stlTypes'
+     */
+    bool isStlType(const std::set<std::string>& stlTypes) const {
+        return isStlType() && stlTypes.find(_start->strAt(2))!=stlTypes.end();
     }
 
     /**
@@ -829,9 +864,10 @@ public:
     /**
      * @brief find a function
      * @param tok token of function call
+     * @param requireConst if const refers to a const variable only const methods should be matched
      * @return pointer to function if found or NULL if not found
      */
-    const Function *findFunction(const Token *tok) const;
+    const Function *findFunction(const Token *tok, bool requireConst=false) const;
 
     /**
      * @brief find if name is in nested list
@@ -841,12 +877,12 @@ public:
 
     const Scope *findRecordInNestedList(const std::string & name) const;
     Scope *findRecordInNestedList(const std::string & name) {
-        return const_cast<Scope *>(static_cast<const Scope *>(this)->findRecordInNestedList(name));
+        return const_cast<Scope *>(const_cast<const Scope *>(this)->findRecordInNestedList(name));
     }
 
     const Type* findType(const std::string& name) const;
     Type* findType(const std::string& name) {
-        return const_cast<Type*>(static_cast<const Scope *>(this)->findType(name));
+        return const_cast<Type*>(const_cast<const Scope *>(this)->findType(name));
     }
 
     /**
@@ -945,19 +981,12 @@ public:
 
     const Type* findType(const Token *tok, const Scope *startScope) const;
     Type* findType(const Token *tok, Scope *startScope) const {
-        return const_cast<Type*>(this->findType(tok, static_cast<const Scope *>(startScope)));
+        return const_cast<Type*>(this->findType(tok, const_cast<const Scope *>(startScope)));
     }
 
     const Scope *findScope(const Token *tok, const Scope *startScope) const;
     Scope *findScope(const Token *tok, Scope *startScope) const {
-        return const_cast<Scope *>(this->findScope(tok, static_cast<const Scope *>(startScope)));
-    }
-
-    bool isClassOrStruct(const std::string &type) const {
-        for (std::list<Type>::const_iterator i = typeList.begin(); i != typeList.end(); ++i)
-            if (i->name() == type)
-                return true;
-        return false;
+        return const_cast<Scope *>(this->findScope(tok, const_cast<const Scope *>(startScope)));
     }
 
     const Variable *getVariableFromVarId(std::size_t varId) const {
@@ -987,11 +1016,14 @@ private:
     Function *addGlobalFunctionDecl(Scope*& scope, const Token* tok, const Token *argStart, const Token* funcStart);
     Function *addGlobalFunction(Scope*& scope, const Token*& tok, const Token *argStart, const Token* funcStart);
     void addNewFunction(Scope **info, const Token **tok);
-    static bool isFunction(const Token *tok, const Scope* outerScope, const Token **funcStart, const Token **argStart);
+    bool isFunction(const Token *tok, const Scope* outerScope, const Token **funcStart, const Token **argStart);
     const Type *findTypeInNested(const Token *tok, const Scope *startScope) const;
     const Scope *findNamespace(const Token * tok, const Scope * scope) const;
     Function *findFunctionInScope(const Token *func, const Scope *ns);
 
+    /** Whether iName is a keyword as defined in http://en.cppreference.com/w/c/keyword and http://en.cppreference.com/w/cpp/keyword*/
+    bool isReservedName(const std::string& iName) const;
+
 
     const Tokenizer *_tokenizer;
     const Settings *_settings;
@@ -1003,5 +1035,6 @@ private:
     /** list for missing types */
     std::list<Type> _blankTypes;
 };
+
 //---------------------------------------------------------------------------
 #endif // symboldatabaseH
