@@ -118,14 +118,16 @@ void TemplateSimplifier::cleanupAfterSimplify(Token *tokens)
 }
 
 
-const Token* TemplateSimplifier::hasComplicatedSyntaxErrorsInTemplates(Token *tokens)
+void TemplateSimplifier::checkComplicatedSyntaxErrorsInTemplates(const Token *tokens)
 {
     // check for more complicated syntax errors when using templates..
     for (const Token *tok = tokens; tok; tok = tok->next()) {
         // skip executing scopes (ticket #3183)..
-        if (Token::simpleMatch(tok, "( {"))
+        if (Token::simpleMatch(tok, "( {")) {
             tok = tok->link();
-
+            if (!tok)
+                syntaxError(nullptr);
+        }
         // skip executing scopes..
         const Token *start = Tokenizer::startOfExecutableScope(tok);
         if (start) {
@@ -142,6 +144,8 @@ const Token* TemplateSimplifier::hasComplicatedSyntaxErrorsInTemplates(Token *to
             }
         }
 
+        if (!tok)
+            syntaxError(nullptr);
         // not start of statement?
         if (tok->previous() && !Token::Match(tok, "[;{}]"))
             continue;
@@ -205,14 +209,10 @@ const Token* TemplateSimplifier::hasComplicatedSyntaxErrorsInTemplates(Token *to
                         --level;
                 }
             }
-            if (level > 0) {
-//                syntaxError(tok);
-                return tok;
-            }
+            if (level > 0)
+                syntaxError(tok);
         }
     }
-
-    return 0;
 }
 
 unsigned int TemplateSimplifier::templateParameters(const Token *tok)
@@ -271,7 +271,7 @@ unsigned int TemplateSimplifier::templateParameters(const Token *tok)
             return 0;
 
         // num/type ..
-        if (!tok->isNumber() && tok->type() != Token::eChar && !tok->isName())
+        if (!tok->isNumber() && tok->tokType() != Token::eChar && !tok->isName())
             return 0;
         tok = tok->next();
         if (!tok)
@@ -286,6 +286,9 @@ unsigned int TemplateSimplifier::templateParameters(const Token *tok)
 
         // Function pointer or prototype..
         while (Token::Match(tok, "(|[")) {
+            if (!tok->link())
+                syntaxError(tok);
+
             tok = tok->link()->next();
             while (Token::Match(tok, "const|volatile")) // Ticket #5786: Skip function cv-qualifiers
                 tok = tok->next();
@@ -562,6 +565,11 @@ void TemplateSimplifier::useDefaultArgumentValues(const std::list<Token *> &temp
                 continue;
             }
 
+            if (tok->str() == "(") { // Ticket #6835
+                tok = tok->link();
+                continue;
+            }
+
             if (tok->str() == "<" && templateParameters(tok))
                 ++templateParmDepth;
 
@@ -569,9 +577,10 @@ void TemplateSimplifier::useDefaultArgumentValues(const std::list<Token *> &temp
             if (tok->str() == ">") {
                 if (Token::Match(tok, "> class|struct %name%"))
                     classname = tok->strAt(2);
-                --templateParmDepth;
-                if (0 == templateParmDepth)
+                if (templateParmDepth<2)
                     break;
+                else
+                    --templateParmDepth;
             }
 
             // next template parameter
@@ -601,7 +610,7 @@ void TemplateSimplifier::useDefaultArgumentValues(const std::list<Token *> &temp
 
             // count the parameters..
             tok = tok->next();
-            unsigned int usedpar = TemplateSimplifier::templateParameters(tok);
+            const unsigned int usedpar = TemplateSimplifier::templateParameters(tok);
             tok = tok->findClosingBracket();
 
             if (tok && tok->str() == ">") {
@@ -927,7 +936,7 @@ bool TemplateSimplifier::simplifyNumericCalculations(Token *tok)
     while (tok->tokAt(4) && tok->next()->isNumber() && tok->tokAt(3)->isNumber()) { // %any% %num% %any% %num% %any%
         const Token* op = tok->tokAt(2);
         const Token* after = tok->tokAt(4);
-        if (Token::Match(tok, "* %num% /") && (tok->strAt(3) != "0") && tok->next()->str() == MathLib::multiply(tok->strAt(3), MathLib::divide(tok->next()->str(), tok->strAt(3)))) {
+        if (Token::Match(tok, "* %num% /") && (op->strAt(1) != "0") && tok->next()->str() == MathLib::multiply(op->strAt(1), MathLib::divide(tok->next()->str(), op->strAt(1)))) {
             // Division where result is a whole number
         } else if (!((op->str() == "*" && (isLowerThanMulDiv(tok) || tok->str() == "*") && isLowerEqualThanMulDiv(after)) || // associative
                      (Token::Match(op, "[/%]") && isLowerThanMulDiv(tok) && isLowerEqualThanMulDiv(after)) || // NOT associative
@@ -1144,7 +1153,7 @@ bool TemplateSimplifier::simplifyCalculations(Token *_tokens)
                         result = (op1 >= op2) ? "1" : "0";
                     else if (cmp == "<")
                         result = (op1 < op2) ? "1" : "0";
-                    else if (cmp == ">")
+                    else
                         result = (op1 > op2) ? "1" : "0";
 
                     tok->str(result);
@@ -1415,7 +1424,7 @@ void TemplateSimplifier::simplifyTemplates(
                 templates2.push_back(*iter1);
         }
 
-        for (std::list<Token *>::iterator it = templates2.begin(); it != templates2.end(); ++it) {
+        for (std::list<Token *>::const_iterator it = templates2.begin(); it != templates2.end(); ++it) {
             std::list<Token *>::iterator it1 = std::find(templates.begin(), templates.end(), *it);
             if (it1 != templates.end()) {
                 templates.erase(it1);
@@ -1424,3 +1433,8 @@ void TemplateSimplifier::simplifyTemplates(
         }
     }
 }
+
+void TemplateSimplifier::syntaxError(const Token *tok)
+{
+    throw InternalError(tok, "syntax error", InternalError::SYNTAX);
+}
