@@ -44,7 +44,7 @@ Token::Token(Token **t) :
     _fileIndex(0),
     _linenr(0),
     _progressValue(0),
-    _type(eNone),
+    _tokType(eNone),
     _flags(0),
     _astOperand1(nullptr),
     _astOperand2(nullptr),
@@ -62,32 +62,32 @@ void Token::update_property_info()
 {
     if (!_str.empty()) {
         if (_str == "true" || _str == "false")
-            _type = eBoolean;
+            _tokType = eBoolean;
         else if (_str[0] == '_' || std::isalpha((unsigned char)_str[0])) { // Name
             if (_varId)
-                _type = eVariable;
-            else if (_type != eVariable && _type != eFunction && _type != eType && _type != eKeyword)
-                _type = eName;
+                _tokType = eVariable;
+            else if (_tokType != eVariable && _tokType != eFunction && _tokType != eType && _tokType != eKeyword)
+                _tokType = eName;
         } else if (std::isdigit((unsigned char)_str[0]) || (_str.length() > 1 && _str[0] == '-' && std::isdigit((unsigned char)_str[1])))
-            _type = eNumber;
-        else if (_str.length() > 1 && _str[0] == '"' && _str[_str.length()-1] == '"')
-            _type = eString;
-        else if (_str.length() > 1 && _str[0] == '\'' && _str[_str.length()-1] == '\'')
-            _type = eChar;
+            _tokType = eNumber;
+        else if (_str.length() > 1 && _str[0] == '"' && _str.back() == '"')
+            _tokType = eString;
+        else if (_str.length() > 1 && _str[0] == '\'' && _str.back() == '\'')
+            _tokType = eChar;
         else if (_str == "=" || _str == "<<=" || _str == ">>=" ||
                  (_str.size() == 2U && _str[1] == '=' && std::strchr("+-*/%&^|", _str[0])))
-            _type = eAssignmentOp;
+            _tokType = eAssignmentOp;
         else if (_str.size() == 1 && _str.find_first_of(",[]()?:") != std::string::npos)
-            _type = eExtendedOp;
+            _tokType = eExtendedOp;
         else if (_str=="<<" || _str==">>" || (_str.size()==1 && _str.find_first_of("+-*/%") != std::string::npos))
-            _type = eArithmeticalOp;
+            _tokType = eArithmeticalOp;
         else if (_str.size() == 1 && _str.find_first_of("&|^~") != std::string::npos)
-            _type = eBitOp;
+            _tokType = eBitOp;
         else if (_str.size() <= 2 &&
                  (_str == "&&" ||
                   _str == "||" ||
                   _str == "!"))
-            _type = eLogicalOp;
+            _tokType = eLogicalOp;
         else if (_str.size() <= 2 && !_link &&
                  (_str == "==" ||
                   _str == "!=" ||
@@ -95,22 +95,27 @@ void Token::update_property_info()
                   _str == "<=" ||
                   _str == ">"  ||
                   _str == ">="))
-            _type = eComparisonOp;
+            _tokType = eComparisonOp;
         else if (_str.size() == 2 &&
                  (_str == "++" ||
                   _str == "--"))
-            _type = eIncDecOp;
+            _tokType = eIncDecOp;
         else if (_str.size() == 1 && (_str.find_first_of("{}") != std::string::npos || (_link && _str.find_first_of("<>") != std::string::npos)))
-            _type = eBracket;
+            _tokType = eBracket;
         else
-            _type = eOther;
+            _tokType = eOther;
     } else {
-        _type = eNone;
+        _tokType = eNone;
     }
 
     update_property_isStandardType();
 }
 
+namespace {
+    static const std::set<std::string> stdTypes = make_container<std::set<std::string> >() <<
+            "bool" << "char" << "char16_t" << "char32_t" << "double" << "float" << "int" << "long" << "short" << "size_t" << "void" << "wchar_t";
+}
+
 void Token::update_property_isStandardType()
 {
     isStandardType(false);
@@ -118,10 +123,9 @@ void Token::update_property_isStandardType()
     if (_str.size() < 3)
         return;
 
-    static const char * const stdtype[] = { "bool", "char", "char16_t", "char32_t", "double", "float", "int", "long", "short", "size_t", "void", "wchar_t"};
-    if (std::binary_search(stdtype, stdtype + sizeof(stdtype) / sizeof(stdtype[0]), _str)) {
+    if (stdTypes.find(_str)!=stdTypes.end()) {
         isStandardType(true);
-        _type = eType;
+        _tokType = eType;
     }
 }
 
@@ -147,8 +151,24 @@ void Token::concatStr(std::string const& b)
 
 std::string Token::strValue() const
 {
-    assert(_type == eString);
-    return _str.substr(1, _str.length() - 2);
+    assert(_tokType == eString);
+    std::string ret(_str.substr(1, _str.length() - 2));
+    std::string::size_type pos = 0U;
+    while ((pos = ret.find('\\', pos)) != std::string::npos) {
+        ret.erase(pos,1U);
+        if (ret[pos] >= 'a') {
+            if (ret[pos] == 'n')
+                ret[pos] = '\n';
+            else if (ret[pos] == 'r')
+                ret[pos] = '\r';
+            else if (ret[pos] == 't')
+                ret[pos] = '\t';
+        }
+        if (ret[pos] == '0')
+            return ret.substr(0,pos);
+        pos++;
+    }
+    return ret;
 }
 
 void Token::deleteNext(unsigned long index)
@@ -172,7 +192,7 @@ void Token::swapWithNext()
         Token temp(0);
 
         temp._str = _next->_str;
-        temp._type = _next->_type;
+        temp._tokType = _next->_tokType;
         temp._flags = _next->_flags;
         temp._varId = _next->_varId;
         temp._fileIndex = _next->_fileIndex;
@@ -184,7 +204,7 @@ void Token::swapWithNext()
         temp._progressValue = _next->_progressValue;
 
         _next->_str = _str;
-        _next->_type = _type;
+        _next->_tokType = _tokType;
         _next->_flags = _flags;
         _next->_varId = _varId;
         _next->_fileIndex = _fileIndex;
@@ -196,7 +216,7 @@ void Token::swapWithNext()
         _next->_progressValue = _progressValue;
 
         _str = temp._str;
-        _type = temp._type;
+        _tokType = temp._tokType;
         _flags = temp._flags;
         _varId = temp._varId;
         _fileIndex = temp._fileIndex;
@@ -213,7 +233,7 @@ void Token::deleteThis()
 {
     if (_next) { // Copy next to this and delete next
         _str = _next->_str;
-        _type = _next->_type;
+        _tokType = _next->_tokType;
         _flags = _next->_flags;
         _varId = _next->_varId;
         _fileIndex = _next->_fileIndex;
@@ -222,6 +242,7 @@ void Token::deleteThis()
         _scope = _next->_scope;
         _function = _next->_function;
         _variable = _next->_variable;
+        _type = _next->_type;
         if (_next->_originalName) {
             _originalName = _next->_originalName;
             _next->_originalName = nullptr;
@@ -233,7 +254,7 @@ void Token::deleteThis()
         deleteNext();
     } else if (_previous && _previous->_previous) { // Copy previous to this and delete previous
         _str = _previous->_str;
-        _type = _previous->_type;
+        _tokType = _previous->_tokType;
         _flags = _previous->_flags;
         _varId = _previous->_varId;
         _fileIndex = _previous->_fileIndex;
@@ -242,6 +263,7 @@ void Token::deleteThis()
         _scope = _previous->_scope;
         _function = _previous->_function;
         _variable = _previous->_variable;
+        _type = _previous->_type;
         if (_previous->_originalName) {
             _originalName = _previous->_originalName;
             _previous->_originalName = nullptr;
@@ -386,7 +408,7 @@ static int multiComparePercent(const Token *tok, const char*& haystack, bool emp
         // Character (%char%)
         if (haystack[0] == 'h') {
             haystack += 4;
-            if (tok->type() == Token::eChar)
+            if (tok->tokType() == Token::eChar)
                 return 1;
         }
         // Const operator (%cop%)
@@ -407,7 +429,7 @@ static int multiComparePercent(const Token *tok, const char*& haystack, bool emp
         // String (%str%)
     {
         haystack += 4;
-        if (tok->type() == Token::eString)
+        if (tok->tokType() == Token::eString)
             return 1;
     }
     break;
@@ -431,7 +453,7 @@ static int multiComparePercent(const Token *tok, const char*& haystack, bool emp
             // Or (%or%)
             else {
                 haystack += 2;
-                if (tok->type() == Token::eBitOp && tok->str() == "|")
+                if (tok->tokType() == Token::eBitOp && tok->str() == "|")
                     return 1;
             }
         }
@@ -439,7 +461,7 @@ static int multiComparePercent(const Token *tok, const char*& haystack, bool emp
         // Oror (%oror%)
         else {
             haystack += 4;
-            if (tok->type() == Token::eLogicalOp && tok->str() == "||")
+            if (tok->tokType() == Token::eLogicalOp && tok->str() == "||")
                 return 1;
         }
     }
@@ -663,21 +685,25 @@ bool Token::Match(const Token *tok, const char pattern[], unsigned int varid)
 std::size_t Token::getStrLength(const Token *tok)
 {
     assert(tok != nullptr);
+    assert(tok->_tokType == eString);
 
     std::size_t len = 0;
-    const std::string strValue(tok->strValue());
-    const char *str = strValue.c_str();
+    std::string::const_iterator it = tok->str().begin() + 1U;
+    const std::string::const_iterator end = tok->str().end() - 1U;
 
-    while (*str) {
-        if (*str == '\\') {
-            ++str;
+    while (it != end) {
+        if (*it == '\\') {
+            ++it;
 
             // string ends at '\0'
-            if (*str == '0')
-                break;
+            if (*it == '0')
+                return len;
         }
 
-        ++str;
+        if (*it == '\0')
+            return len;
+
+        ++it;
         ++len;
     }
 
@@ -686,7 +712,7 @@ std::size_t Token::getStrLength(const Token *tok)
 
 std::size_t Token::getStrSize(const Token *tok)
 {
-    assert(tok != nullptr && tok->type() == eString);
+    assert(tok != nullptr && tok->tokType() == eString);
     const std::string &str = tok->str();
     unsigned int sizeofstring = 1U;
     for (unsigned int i = 1U; i < str.size() - 1U; i++) {
@@ -796,9 +822,11 @@ const Token * Token::findClosingBracket() const
     if (_str == "<") {
         unsigned int depth = 0;
         for (closing = this; closing != nullptr; closing = closing->next()) {
-            if (Token::Match(closing, "{|[|("))
+            if (Token::Match(closing, "{|[|(")) {
                 closing = closing->link();
-            else if (Token::Match(closing, "}|]|)|;"))
+                if (!closing)
+                    return nullptr; // #6803
+            } else if (Token::Match(closing, "}|]|)|;"))
                 break;
             else if (closing->str() == "<")
                 ++depth;
@@ -825,36 +853,36 @@ Token * Token::findClosingBracket()
 
 //---------------------------------------------------------------------------
 
-const Token *Token::findsimplematch(const Token *tok, const char pattern[])
+const Token *Token::findsimplematch(const Token *startTok, const char pattern[])
 {
-    for (; tok; tok = tok->next()) {
+    for (const Token* tok = startTok; tok; tok = tok->next()) {
         if (Token::simpleMatch(tok, pattern))
             return tok;
     }
     return 0;
 }
 
-const Token *Token::findsimplematch(const Token *tok, const char pattern[], const Token *end)
+const Token *Token::findsimplematch(const Token *startTok, const char pattern[], const Token *end)
 {
-    for (; tok && tok != end; tok = tok->next()) {
+    for (const Token* tok = startTok; tok && tok != end; tok = tok->next()) {
         if (Token::simpleMatch(tok, pattern))
             return tok;
     }
     return 0;
 }
 
-const Token *Token::findmatch(const Token *tok, const char pattern[], unsigned int varId)
+const Token *Token::findmatch(const Token *startTok, const char pattern[], unsigned int varId)
 {
-    for (; tok; tok = tok->next()) {
+    for (const Token* tok = startTok; tok; tok = tok->next()) {
         if (Token::Match(tok, pattern, varId))
             return tok;
     }
     return 0;
 }
 
-const Token *Token::findmatch(const Token *tok, const char pattern[], const Token *end, unsigned int varId)
+const Token *Token::findmatch(const Token *startTok, const char pattern[], const Token *end, unsigned int varId)
 {
-    for (; tok && tok != end; tok = tok->next()) {
+    for (const Token* tok = startTok; tok && tok != end; tok = tok->next()) {
         if (Token::Match(tok, pattern, varId))
             return tok;
     }
@@ -980,8 +1008,10 @@ void Token::stringify(std::ostream& os, bool varid, bool attributes, bool macro)
             os << "unsigned ";
         else if (isSigned())
             os << "signed ";
+        if (isComplex())
+            os << "_Complex ";
         if (isLong()) {
-            if (_type == eString || _type == eChar)
+            if (_tokType == eString || _tokType == eChar)
                 os << "L";
             else
                 os << "long ";
@@ -989,7 +1019,7 @@ void Token::stringify(std::ostream& os, bool varid, bool attributes, bool macro)
     }
     if (macro && isExpandedMacro())
         os << "$";
-    if (_str[0] != '\"' || _str.find("\0") == std::string::npos)
+    if (_str[0] != '\"' || _str.find('\0') == std::string::npos)
         os << _str;
     else {
         for (std::size_t i = 0U; i < _str.size(); ++i) {
@@ -1073,12 +1103,16 @@ std::string Token::stringifyList(bool varid) const
 
 void Token::astOperand1(Token *tok)
 {
+    const Token* const root = tok;
     if (_astOperand1)
         _astOperand1->_astParent = nullptr;
     // goto parent operator
     if (tok) {
-        while (tok->_astParent)
+        while (tok->_astParent) {
+            if (tok->_astParent == this || tok->_astParent == root) // #6838/#6726 avoid hang on garbage code
+                throw InternalError(this, "Internal error. Token::astOperand1() cyclic dependency.");
             tok = tok->_astParent;
+        }
         tok->_astParent = this;
     }
     _astOperand1 = tok;
@@ -1086,19 +1120,35 @@ void Token::astOperand1(Token *tok)
 
 void Token::astOperand2(Token *tok)
 {
+    const Token* const root = tok;
     if (_astOperand2)
         _astOperand2->_astParent = nullptr;
     // goto parent operator
     if (tok) {
-        while (tok->_astParent)
+        while (tok->_astParent) {
+            //std::cout << tok << " -> " << tok->_astParent ;
+            if (tok->_astParent == this || tok->_astParent == root) // #6838/#6726 avoid hang on garbage code
+                throw InternalError(this, "Internal error. Token::astOperand2() cyclic dependency.");
             tok = tok->_astParent;
+        }
         tok->_astParent = this;
     }
     _astOperand2 = tok;
 }
 
-bool Token::isCalculation() const
+bool Token::isCalculation(bool goDownwards) const
 {
+    if (goDownwards && Token::Match(this, "[|(|,")) {
+        bool ret = false;
+        if (this->astOperand1())
+            ret = this->astOperand1()->isCalculation(true);
+        if (ret)
+            return true;
+        if (this->astOperand2())
+            ret = this->astOperand2()->isCalculation(true);
+        return ret;
+
+    }
     if (!Token::Match(this, "%cop%|++|--"))
         return false;
 
@@ -1133,18 +1183,21 @@ bool Token::isCalculation() const
     return true;
 }
 
-static bool isUnaryPreOp(const Token *op)
+bool Token::isUnaryPreOp() const
 {
-    if (!op->astOperand1() || op->astOperand2())
+    if (!astOperand1() || astOperand2())
         return false;
-    if (!Token::Match(op, "++|--"))
+    if (!Token::Match(this, "++|--"))
         return true;
-    const Token *tok = op->astOperand1();
+    const Token *tokbefore = _previous;
+    const Token *tokafter = _next;
     for (int distance = 1; distance < 10; distance++) {
-        if (tok == op->tokAt(-distance))
+        if (tokbefore == _astOperand1)
             return false;
-        if (tok == op->tokAt(distance))
+        if (tokafter == _astOperand1)
             return true;
+        tokbefore = tokbefore->_previous;
+        tokafter  = tokafter->_previous;
     }
     return false; // <- guess
 }
@@ -1153,16 +1206,44 @@ std::string Token::expressionString() const
 {
     const Token * const top = this;
     const Token *start = top;
-    while (start->astOperand1() && start->astOperand2())
+    while (start->astOperand1() &&
+           (start->astOperand2() || !start->isUnaryPreOp() || Token::simpleMatch(start, "( )")))
         start = start->astOperand1();
     const Token *end = top;
-    while (end->astOperand1() && (end->astOperand2() || isUnaryPreOp(end))) {
+    while (end->astOperand1() && (end->astOperand2() || end->isUnaryPreOp())) {
         if (Token::Match(end,"(|[")) {
             end = end->link();
             break;
         }
         end = end->astOperand2() ? end->astOperand2() : end->astOperand1();
     }
+
+    // move start to lpar in such expression: '(*it).x'
+    int par = 0;
+    for (const Token *tok = start; tok != end; tok = tok->next()) {
+        if (tok->str() == "(")
+            ++par;
+        else if (tok->str() == ")") {
+            if (par == 0)
+                start = tok->link();
+            else
+                --par;
+        }
+    }
+
+    // move end to rpar in such expression: '2>(x+1)'
+    par = 0;
+    for (const Token *tok = end; tok != start; tok = tok->previous()) {
+        if (tok->str() == ")")
+            ++par;
+        else if (tok->str() == "(") {
+            if (par == 0)
+                end = tok->link();
+            else
+                --par;
+        }
+    }
+
     std::string ret;
     for (const Token *tok = start; tok && tok != end; tok = tok->next()) {
         ret += tok->str();
@@ -1170,7 +1251,6 @@ std::string Token::expressionString() const
             ret += " ";
     }
     return ret + end->str();
-
 }
 
 static void astStringXml(const Token *tok, std::size_t indent, std::ostream &out)
@@ -1203,28 +1283,26 @@ static void astStringXml(const Token *tok, std::size_t indent, std::ostream &out
 
 void Token::printAst(bool verbose, bool xml, std::ostream &out) const
 {
-    bool title = false;
-
-    bool print = true;
+    std::set<const Token *> printed;
     for (const Token *tok = this; tok; tok = tok->next()) {
-        if (print && tok->_astOperand1) {
-            if (!title && !xml)
+        if (!tok->_astParent && tok->_astOperand1) {
+            if (printed.empty() && !xml)
                 out << "\n\n##AST" << std::endl;
-            title = true;
+            else if (printed.find(tok) != printed.end())
+                continue;
+            printed.insert(tok);
+
             if (xml) {
                 out << "<ast scope=\"" << tok->scope() << "\" fileIndex=\"" << tok->fileIndex() << "\" linenr=\"" << tok->linenr() << "\">" << std::endl;
-                astStringXml(tok->astTop(), 2U, out);
+                astStringXml(tok, 2U, out);
                 out << "</ast>" << std::endl;
             } else if (verbose)
-                out << tok->astTop()->astStringVerbose(0,0) << std::endl;
+                out << tok->astStringVerbose(0,0) << std::endl;
             else
-                out << tok->astTop()->astString(" ") << std::endl;
-            print = false;
+                out << tok->astString(" ") << std::endl;
             if (tok->str() == "(")
                 tok = tok->link();
         }
-        if (Token::Match(tok, "[;{}]"))
-            print = true;
     }
 }
 
@@ -1275,8 +1353,11 @@ void Token::printValueFlow(bool xml, std::ostream &out) const
         else if (line != tok->linenr())
             out << "Line " << tok->linenr() << std::endl;
         line = tok->linenr();
-        if (!xml)
-            out << "  " << tok->str() << ":{";
+        if (!xml) {
+            out << "  " << tok->str() << (tok->values.front().isKnown() ? " always " : " possible ");
+            if (tok->values.size() > 1U)
+                out << '{';
+        }
         for (std::list<ValueFlow::Value>::const_iterator it=tok->values.begin(); it!=tok->values.end(); ++it) {
             if (xml) {
                 out << "      <value ";
@@ -1286,6 +1367,10 @@ void Token::printValueFlow(bool xml, std::ostream &out) const
                     out << "intvalue=\"" << it->intvalue << '\"';
                 if (it->condition)
                     out << " condition-line=\"" << it->condition->linenr() << '\"';
+                if (it->isKnown())
+                    out << " known=\"true\"";
+                else if (it->isPossible())
+                    out << " possible=\"true\"";
                 out << "/>" << std::endl;
             }
 
@@ -1300,8 +1385,10 @@ void Token::printValueFlow(bool xml, std::ostream &out) const
         }
         if (xml)
             out << "    </values>" << std::endl;
+        else if (tok->values.size() > 1U)
+            out << '}' << std::endl;
         else
-            out << "}" << std::endl;
+            out << std::endl;
     }
     if (xml)
         out << "  </valueflow>" << std::endl;
@@ -1355,7 +1442,7 @@ const Token *Token::getValueTokenMinStrSize() const
     std::size_t minsize = ~0U;
     std::list<ValueFlow::Value>::const_iterator it;
     for (it = values.begin(); it != values.end(); ++it) {
-        if (it->tokvalue && it->tokvalue->type() == Token::eString) {
+        if (it->tokvalue && it->tokvalue->tokType() == Token::eString) {
             std::size_t size = getStrSize(it->tokvalue);
             if (!ret || size < minsize) {
                 minsize = size;
@@ -1372,7 +1459,7 @@ const Token *Token::getValueTokenMaxStrLength() const
     std::size_t maxlength = 0U;
     std::list<ValueFlow::Value>::const_iterator it;
     for (it = values.begin(); it != values.end(); ++it) {
-        if (it->tokvalue && it->tokvalue->type() == Token::eString) {
+        if (it->tokvalue && it->tokvalue->tokType() == Token::eString) {
             std::size_t length = getStrLength(it->tokvalue);
             if (!ret || length > maxlength) {
                 maxlength = length;
@@ -1406,6 +1493,8 @@ const Token *Token::getValueTokenDeadPointer() const
         const Variable * const var = vartok->variable();
         if (var->isStatic() || var->isReference())
             continue;
+        if (!var->scope())
+            return nullptr; // #6804
         if (var->scope()->type == Scope::eUnion && var->scope()->nestedIn == this->scope())
             continue;
         // variable must be in same function (not in subfunction)
