@@ -30,6 +30,7 @@
 #include "mathlib.h"
 
 class Scope;
+class Type;
 class Function;
 class Variable;
 class Settings;
@@ -59,7 +60,7 @@ private:
 public:
     enum Type {
         eVariable, eType, eFunction, eKeyword, eName, // Names: Variable (varId), Type (typeId, later), Function (FuncId, later), Language keyword, Name (unknown identifier)
-        eNumber, eString, eChar, eBoolean, eLiteral, // Literals: Number, String, Character, User defined literal (C++11)
+        eNumber, eString, eChar, eBoolean, eLiteral, // Literals: Number, String, Character, Boolean, User defined literal (C++11)
         eArithmeticalOp, eComparisonOp, eAssignmentOp, eLogicalOp, eBitOp, eIncDecOp, eExtendedOp, // Operators: Arithmetical, Comparison, Assignment, Logical, Bitwise, ++/--, Extended
         eBracket, // {, }, <, >: < and > only if link() is set. Otherwise they are comparison operators.
         eOther,
@@ -104,7 +105,7 @@ public:
      */
     const Token *tokAt(int index) const;
     Token *tokAt(int index) {
-        return const_cast<Token *>(static_cast<const Token *>(this)->tokAt(index));
+        return const_cast<Token *>(const_cast<const Token *>(this)->tokAt(index));
     }
 
     /**
@@ -113,7 +114,7 @@ public:
      */
     const Token *linkAt(int index) const;
     Token *linkAt(int index) {
-        return const_cast<Token *>(static_cast<const Token *>(this)->linkAt(index));
+        return const_cast<Token *>(const_cast<const Token *>(this)->linkAt(index));
     }
 
     /**
@@ -222,60 +223,61 @@ public:
      **/
     static std::string getCharAt(const Token *tok, std::size_t index);
 
-    Type type() const {
-        return _type;
+    Token::Type tokType() const {
+        return _tokType;
     }
-    void type(Type t) {
-        _type = t;
+    void tokType(Token::Type t) {
+        _tokType = t;
     }
     void isKeyword(bool kwd) {
         if (kwd)
-            _type = eKeyword;
-        else if (_type == eKeyword)
-            _type = eName;
+            _tokType = eKeyword;
+        else if (_tokType == eKeyword)
+            _tokType = eName;
     }
     bool isKeyword() const {
-        return _type == eKeyword;
+        return _tokType == eKeyword;
     }
     bool isName() const {
-        return _type == eName || _type == eType || _type == eVariable || _type == eFunction || _type == eKeyword ||
-               _type == eBoolean; // TODO: "true"/"false" aren't really a name...
+        return _tokType == eName || _tokType == eType || _tokType == eVariable || _tokType == eFunction || _tokType == eKeyword ||
+               _tokType == eBoolean; // TODO: "true"/"false" aren't really a name...
     }
     bool isUpperCaseName() const;
     bool isLiteral() const {
-        return _type == eNumber || _type == eString || _type == eChar ||
-               _type == eBoolean || _type == eLiteral;
+        return _tokType == eNumber || _tokType == eString || _tokType == eChar ||
+               _tokType == eBoolean || _tokType == eLiteral;
     }
     bool isNumber() const {
-        return _type == eNumber;
+        return _tokType == eNumber;
     }
     bool isOp() const {
         return (isConstOp() ||
                 isAssignmentOp() ||
-                _type == eIncDecOp);
+                _tokType == eIncDecOp);
     }
     bool isConstOp() const {
         return (isArithmeticalOp() ||
-                _type == eLogicalOp ||
-                _type == eComparisonOp ||
-                _type == eBitOp);
+                _tokType == eLogicalOp ||
+                _tokType == eComparisonOp ||
+                _tokType == eBitOp);
     }
     bool isExtendedOp() const {
         return isConstOp() ||
-               _type == eExtendedOp;
+               _tokType == eExtendedOp;
     }
     bool isArithmeticalOp() const {
-        return _type == eArithmeticalOp;
+        return _tokType == eArithmeticalOp;
     }
     bool isComparisonOp() const {
-        return _type == eComparisonOp;
+        return _tokType == eComparisonOp;
     }
     bool isAssignmentOp() const {
-        return _type == eAssignmentOp;
+        return _tokType == eAssignmentOp;
     }
     bool isBoolean() const {
-        return _type == eBoolean;
+        return _tokType == eBoolean;
     }
+    bool isUnaryPreOp() const;
 
     unsigned int flags() const {
         return _flags;
@@ -373,22 +375,34 @@ public:
     void isAttributeNothrow(bool value) {
         setFlag(fIsAttributeNothrow, value);
     }
+    bool isOperatorKeyword() const {
+        return getFlag(fIsOperatorKeyword);
+    }
+    void isOperatorKeyword(bool value) {
+        setFlag(fIsOperatorKeyword, value);
+    }
+    bool isComplex() const {
+        return getFlag(fIsComplex);
+    }
+    void isComplex(bool value) {
+        setFlag(fIsComplex, value);
+    }
 
-    static const Token *findsimplematch(const Token *tok, const char pattern[]);
-    static const Token *findsimplematch(const Token *tok, const char pattern[], const Token *end);
-    static const Token *findmatch(const Token *tok, const char pattern[], unsigned int varId = 0);
-    static const Token *findmatch(const Token *tok, const char pattern[], const Token *end, unsigned int varId = 0);
-    static Token *findsimplematch(Token *tok, const char pattern[]) {
-        return const_cast<Token *>(findsimplematch(static_cast<const Token *>(tok), pattern));
+    static const Token *findsimplematch(const Token *startTok, const char pattern[]);
+    static const Token *findsimplematch(const Token *startTok, const char pattern[], const Token *end);
+    static const Token *findmatch(const Token *startTok, const char pattern[], unsigned int varId = 0);
+    static const Token *findmatch(const Token *startTok, const char pattern[], const Token *end, unsigned int varId = 0);
+    static Token *findsimplematch(Token *startTok, const char pattern[]) {
+        return const_cast<Token *>(findsimplematch(const_cast<const Token *>(startTok), pattern));
     }
-    static Token *findsimplematch(Token *tok, const char pattern[], const Token *end) {
-        return const_cast<Token *>(findsimplematch(static_cast<const Token *>(tok), pattern, end));
+    static Token *findsimplematch(Token *startTok, const char pattern[], const Token *end) {
+        return const_cast<Token *>(findsimplematch(const_cast<const Token *>(startTok), pattern, end));
     }
-    static Token *findmatch(Token *tok, const char pattern[], unsigned int varId = 0) {
-        return const_cast<Token *>(findmatch(static_cast<const Token *>(tok), pattern, varId));
+    static Token *findmatch(Token *startTok, const char pattern[], unsigned int varId = 0) {
+        return const_cast<Token *>(findmatch(const_cast<const Token *>(startTok), pattern, varId));
     }
-    static Token *findmatch(Token *tok, const char pattern[], const Token *end, unsigned int varId = 0) {
-        return const_cast<Token *>(findmatch(static_cast<const Token *>(tok), pattern, end, varId));
+    static Token *findmatch(Token *startTok, const char pattern[], const Token *end, unsigned int varId = 0) {
+        return const_cast<Token *>(findmatch(const_cast<const Token *>(startTok), pattern, end, varId));
     }
 
     /**
@@ -457,7 +471,7 @@ public:
     void varId(unsigned int id) {
         _varId = id;
         if (id != 0)
-            _type = eVariable;
+            _tokType = eVariable;
         else
             update_property_info();
     }
@@ -569,16 +583,16 @@ public:
     void function(const Function *f) {
         _function = f;
         if (f)
-            _type = eFunction;
-        else if (_type == eFunction)
-            _type = eName;
+            _tokType = eFunction;
+        else if (_tokType == eFunction)
+            _tokType = eName;
     }
 
     /**
      * @return a pointer to the Function associated with this token.
      */
     const Function *function() const {
-        return _type == eFunction ? _function : 0;
+        return _tokType == eFunction ? _function : 0;
     }
 
     /**
@@ -588,16 +602,35 @@ public:
     void variable(const Variable *v) {
         _variable = v;
         if (v || _varId)
-            _type = eVariable;
-        else if (_type == eVariable)
-            _type = eName;
+            _tokType = eVariable;
+        else if (_tokType == eVariable)
+            _tokType = eName;
     }
 
     /**
      * @return a pointer to the variable associated with this token.
      */
     const Variable *variable() const {
-        return _type == eVariable ? _variable : 0;
+        return _tokType == eVariable ? _variable : 0;
+    }
+
+    /**
+    * Associate this token with given type
+    * @param t Type to be associated
+    */
+    void type(const ::Type *t) {
+        _type = t;
+        if (t)
+            _tokType = eType;
+        else if (_tokType == eType)
+            _tokType = eName;
+    }
+
+    /**
+    * @return a pointer to the type associated with this token.
+    */
+    const ::Type *type() const {
+        return _tokType == eType ? _type : 0;
     }
 
     /**
@@ -744,6 +777,7 @@ private:
     union {
         const Function *_function;
         const Variable *_variable;
+        const ::Type* _type;
     };
 
     unsigned int _varId;
@@ -756,7 +790,7 @@ private:
      */
     unsigned int _progressValue;
 
-    Type _type;
+    Token::Type _tokType;
 
     enum {
         fIsUnsigned             = (1 << 0),
@@ -773,7 +807,9 @@ private:
         fIsAttributeConst       = (1 << 11), // __attribute__((const))
         fIsAttributeNoreturn    = (1 << 12), // __attribute__((noreturn)), __declspec(noreturn)
         fIsAttributeNothrow     = (1 << 13), // __attribute__((nothrow)), __declspec(nothrow)
-        fIsAttributeUsed        = (1 << 14)  // __attribute__((used))
+        fIsAttributeUsed        = (1 << 14), // __attribute__((used))
+        fIsOperatorKeyword      = (1 << 15), // operator=, etc
+        fIsComplex              = (1 << 16)  // complex/_Complex type
     };
 
     unsigned int _flags;
@@ -836,9 +872,10 @@ public:
      * For '*' and '&' tokens it is looked up if this is a
      * dereference or address-of. A dereference or address-of is not
      * counted as a calculation.
+     * @param goDownwards the function will look for calculations in all children of the tree
      * @return returns true if current token is a calculation
      */
-    bool isCalculation() const;
+    bool isCalculation(bool goDownwards = false) const;
 
     void clearAst() {
         _astOperand1 = _astOperand2 = _astParent = NULL;
