@@ -40,6 +40,10 @@ class TimerResults;
 
 /** @brief The main purpose is to tokenize the source code. It also has functions that simplify the token list */
 class CPPCHECKLIB Tokenizer {
+
+    friend class TestSimplifyTokens;
+    friend class TestSimplifyTypedef;
+    friend class TestTokenizer;
 public:
     Tokenizer();
     Tokenizer(const Settings * settings, ErrorLogger *errorLogger);
@@ -114,6 +118,8 @@ public:
     */
     bool simplifyTokenList1(const char FileName[]);
 
+    void SimplifyNamelessRValueReferences();
+
     /**
     * Most aggressive simplification of tokenlist
     *
@@ -186,6 +192,12 @@ public:
      */
     void simplifyOffsetPointerDereference();
 
+    /**
+       * Simplify referencing a pointer offset:
+       *     "Replace "&str[num]" => "(str + num)"
+       */
+    void simplifyOffsetPointerReference();
+
     /** Insert array size where it isn't given */
     void arraySize();
 
@@ -265,6 +277,12 @@ public:
      */
     void simplifyCompoundAssignment();
 
+
+    /**
+     * Simplify "* const" to "*"
+     */
+    void simplifyPointerConst();
+
     /**
      * Simplify the location of "static" and "const" qualifiers in
      * a variable declaration or definition.
@@ -287,22 +305,6 @@ public:
      */
     void simplifyVariableMultipleAssign();
 
-    /**
-     * simplify if-not
-     * Example: "if(0==x);" => "if(!x);"
-     */
-    void simplifyIfNot();
-
-    /**
-     * simplify if-not NULL
-     * Example: "if(0!=x);" => "if(x);"
-     * Special case: 'x = (0 != x);' is removed.
-     */
-    void simplifyIfNotNull();
-
-    /** @brief simplify if (a) { if (a) */
-    void simplifyIfSameInnerCondition();
-
     /**
      * Simplify the 'C Alternative Tokens'
      * Examples:
@@ -432,9 +434,6 @@ public:
      */
     bool simplifyFunctionReturn();
 
-    /** Struct initialization */
-    void simplifyStructInit();
-
     /** Struct simplification
      * "struct S { } s;" => "struct S { }; S s;"
      */
@@ -494,13 +493,9 @@ public:
 
     void simplifyRoundCurlyParentheses();
 
-    void simplifyDebugNew();
-
     void simplifySQL();
 
-    bool hasEnumsWithTypedef();
-
-    void simplifyDefaultAndDeleteInsideClass();
+    void checkForEnumsWithTypedef();
 
     void findComplicatedSyntaxErrorsInTemplates();
 
@@ -522,10 +517,15 @@ public:
      */
     static std::string simplifyString(const std::string &source);
 
+private:
+
     /**
-     * Change "int const x;" into "const int x;"
+     * is token pointing at function head?
+     * @param tok         A '(' or ')' token in a possible function head
+     * @param endsWith    string after function head
+     * @return token matching with endsWith if syntax seems to be a function head else nullptr
      */
-    void simplifyConst();
+    const Token * isFunctionHead(const Token *tok, const std::string &endsWith) const;
 
     /**
      * simplify "while (0)"
@@ -580,12 +580,16 @@ public:
      */
     void createLinks2();
 
+public:
+
     /** Syntax error */
     void syntaxError(const Token *tok) const;
 
     /** Syntax error. Example: invalid number of ')' */
     void syntaxError(const Token *tok, char c) const;
 
+private:
+
     /** Report that there is an unhandled "class x y {" code */
     void unhandled_macro_class_x_y(const Token *tok) const;
 
@@ -620,16 +624,16 @@ public:
      */
     void simplifyAsm();
 
+    /**
+     * asm heuristics, Put ^{} statements in asm()
+     */
+    void simplifyAsm2();
+
     /**
      * Simplify bitfields - the field width is removed as we don't use it.
      */
     void simplifyBitfields();
 
-    /**
-     * Remove __builtin_expect(...), likely(...), and unlikely(...)
-     */
-    void simplifyBuiltinExpect();
-
     /**
      * Remove unnecessary member qualification
      */
@@ -645,11 +649,6 @@ public:
      */
     void simplifyNamespaceStd();
 
-    /**
-     * Remove Microsoft MFC 'DECLARE_MESSAGE_MAP()'
-     */
-    void simplifyMicrosoftMFC();
-
     /**
     * Convert Microsoft memory functions
     * CopyMemory(dst, src, len) -> memcpy(dst, src, len)
@@ -681,6 +680,17 @@ public:
      */
     void simplifyOperatorName();
 
+    /**
+    * Remove [[deprecated]] (C++14) from TokenList
+    */
+    void simplifyDeprecated();
+
+    /**
+     * Replace strlen(str)
+     * @return true if any replacement took place, false else
+     * */
+    bool simplifyStrlen();
+
     /**
      * check for duplicate enum definition
      */
@@ -707,11 +717,14 @@ public:
 
     void unsupportedTypedef(const Token *tok) const;
 
+public:
+
     /** Was there templates in the code? */
     bool codeWithTemplates() const {
         return _codeWithTemplates;
     }
 
+
     void setSettings(const Settings *settings) {
         _settings = settings;
         list.setSettings(settings);
@@ -723,7 +736,12 @@ public:
     void createSymbolDatabase();
     void deleteSymbolDatabase();
 
-    void printDebugOutput() const;
+    /** print --debug output if debug flags match the simplification:
+     * 0=unknown/both simplifications
+     * 1=1st simplifications
+     * 2=2nd simplifications
+     */
+    void printDebugOutput(unsigned int simplification) const;
 
     void dump(std::ostream &out) const;
 
@@ -737,6 +755,7 @@ public:
         return _varId;
     }
 
+
     /**
      * Simplify e.g. 'return(strncat(temp,"a",1));' into
      * strncat(temp,"a",1); return temp;
@@ -804,11 +823,13 @@ private:
     /** Disable assignment operator, no implementation */
     Tokenizer &operator=(const Tokenizer &);
 
-    static Token * startOfFunction(Token * tok);
+    Token * startOfFunction(Token * tok) const;
     static Token * startOfExecutableScope(Token * tok) {
         return const_cast<Token*>(startOfExecutableScope(const_cast<const Token *>(tok)));
     }
 
+    Token *processFunc(Token *tok2, bool inOperator) const;
+
     /** Set pod types */
     void setPodTypes();
 
