@@ -82,11 +82,6 @@ private:
               "}");
         ASSERT_EQUALS("[test.cpp:5]: (warning) Class x is not safe, destructor throws exception\n", errout.str());
 
-        check("x::~x() {\n"
-              "    throw e;\n"
-              "}");
-        TODO_ASSERT_EQUALS("[test.cpp:3]: (warning) Class x is not safe, destructor throws exception\n", "", errout.str());
-
         // #3858 - throwing exception in try block in destructor.
         check("class x {\n"
               "    ~x() {\n"
