@@ -39,6 +39,7 @@ private:
         TEST_CASE(function_arg_any);
         TEST_CASE(function_arg_valid);
         TEST_CASE(function_arg_minsize);
+        TEST_CASE(function_namespace);
         TEST_CASE(memory);
         TEST_CASE(memory2); // define extra "free" allocation functions
         TEST_CASE(resource);
@@ -288,6 +289,37 @@ private:
         }
     }
 
+    void function_namespace() const {
+        const char xmldata[] = "<?xml version=\"1.0\"?>\n"
+                               "<def>\n"
+                               "  <function name=\"Foo::foo,bar\">\n"
+                               "    <noreturn>false</noreturn>\n"
+                               "  </function>\n"
+                               "</def>";
+        tinyxml2::XMLDocument doc;
+        doc.Parse(xmldata, sizeof(xmldata));
+
+        Library library;
+        library.load(doc);
+        ASSERT(library.use.empty());
+        ASSERT(library.leakignore.empty());
+        ASSERT(library.argumentChecks.empty());
+
+        {
+            TokenList tokenList(nullptr);
+            std::istringstream istr("Foo::foo();");
+            tokenList.createTokens(istr);
+            ASSERT(library.isnotnoreturn(tokenList.front()->tokAt(2)));
+        }
+
+        {
+            TokenList tokenList(nullptr);
+            std::istringstream istr("bar();");
+            tokenList.createTokens(istr);
+            ASSERT(library.isnotnoreturn(tokenList.front()));
+        }
+    }
+
     void memory() const {
         const char xmldata[] = "<?xml version=\"1.0\"?>\n"
                                "<def>\n"
@@ -366,7 +398,7 @@ private:
 
         const struct Library::PodType *type = library.podtype("s16");
         ASSERT_EQUALS(2U,   type ? type->size : 0U);
-        ASSERT_EQUALS(0,    type ? type->sign : '?');
+        ASSERT_EQUALS((char)0,    type ? type->sign : '?');
     }
 
     void container() const {
