@@ -108,6 +108,18 @@ private:
 
         check("sizeof(--foo)");
         ASSERT_EQUALS("[test.cpp:1]: (warning) Found calculation inside sizeof().\n", errout.str());
+
+        check("sizeof(bar(1, 2, --foo, 3, 4))");
+        ASSERT_EQUALS("[test.cpp:1]: (warning) Found calculation inside sizeof().\n", errout.str());
+
+        check("sizeof( int32_t[ i++ ] );");
+        ASSERT_EQUALS("[test.cpp:1]: (warning) Found calculation inside sizeof().\n", errout.str());
+
+        check("sizeof(a[b + 2])");
+        ASSERT_EQUALS("[test.cpp:1]: (warning) Found calculation inside sizeof().\n", errout.str());
+
+        check("sizeof((2 + a)[b])");
+        ASSERT_EQUALS("[test.cpp:1]: (warning) Found calculation inside sizeof().\n", errout.str());
     }
 
     void sizeofForArrayParameter() {
@@ -181,6 +193,14 @@ private:
         ASSERT_EQUALS("[test.cpp:2]: (warning) Using 'sizeof' on array given as "
                       "function argument returns size of a pointer.\n", errout.str());
 
+        check("typedef char Fixname[1000];\n"
+              "int f2(Fixname& f2v) {\n"
+              "  int i = sizeof(f2v);\n"
+              "  printf(\"sizeof f2v %d\n\", i);\n"
+              "   }\n"
+             );
+        ASSERT_EQUALS("", errout.str());
+
         check("void f(int *p) {\n"
               "    p[0] = 0;\n"
               "    int unused = sizeof(p);\n"
