@@ -32,6 +32,8 @@ public:
 private:
 
     void run() {
+        TEST_CASE(stringLiteralWrite);
+
         TEST_CASE(alwaysTrueFalseStringCompare);
         TEST_CASE(suspiciousStringCompare);
         TEST_CASE(suspiciousStringCompare_char);
@@ -78,7 +80,7 @@ private:
         settings.addEnabled("warning");
 
         // Preprocess file..
-        Preprocessor preprocessor(&settings, this);
+        Preprocessor preprocessor(settings, this);
         std::list<std::string> configurations;
         std::string filedata;
         std::istringstream fin(precode);
@@ -95,6 +97,26 @@ private:
         checkString.checkAlwaysTrueOrFalseStringCompare();
     }
 
+    void stringLiteralWrite() {
+        check("void f() {\n"
+              "  char *abc = \"abc\";\n"
+              "  abc[0] = 'a';\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3] -> [test.cpp:2]: (error) Modifying string literal \"abc\" directly or indirectly is undefined behaviour\n", errout.str());
+
+        check("void f() {\n"
+              "  char *abc = \"abc\";\n"
+              "  *abc = 'a';\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3] -> [test.cpp:2]: (error) Modifying string literal \"abc\" directly or indirectly is undefined behaviour\n", errout.str());
+
+        check("void f() {\n"
+              "  QString abc = \"abc\";\n"
+              "  abc[0] = 'a';\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+    }
+
     void alwaysTrueFalseStringCompare() {
         check("void f() {\n"
               "  if (strcmp(\"A\",\"A\")){}\n"
