@@ -75,12 +75,13 @@ private:
     }
 
 
-    void check(const char code[]) {
+    void check(const char code[], Settings::PlatformType platform = Settings::Unspecified) {
         // Clear the error buffer..
         errout.str("");
 
         Settings settings;
         settings.addEnabled("style");
+        settings.platform(platform);
 
         // Tokenize..
         Tokenizer tokenizer(&settings, this);
@@ -517,7 +518,7 @@ private:
               "public:\n"
               "    Foo() { }\n"
               "    __property int x = {read=getx}\n"
-              "};");
+              "};", Settings::Win32A);
         ASSERT_EQUALS("", errout.str());
     }
 
@@ -530,7 +531,7 @@ private:
               "    }\n"
               "public:\n"
               "    Foo() { }\n"
-              "};");
+              "};", Settings::Win32A);
         ASSERT_EQUALS("", errout.str());
     }
 
@@ -721,7 +722,7 @@ private:
               "    C a;\n"
               "}");
 
-        ASSERT_EQUALS("[test.cpp:10]: (style) Unused private function: 'InfiniteA::foo'\n", errout.str());
+        ASSERT_EQUALS("", errout.str());
     }
 
     void staticVariable() {
