@@ -26,6 +26,7 @@
 #include <sstream>
 
 #include "../cli/filelister.h"
+#include "../cli/pathmatch.h"
 
 static std::string builddir(std::string filename)
 {
@@ -91,7 +92,9 @@ static void getCppFiles(std::vector<std::string> &files, const std::string &path
 {
     std::map<std::string,size_t> filemap;
     const std::set<std::string> extra;
-    FileLister::addFiles(filemap, path, extra, recursive);
+    const std::vector<std::string> masks;
+    const PathMatch matcher(masks);
+    FileLister::addFiles(filemap, path, extra, recursive, matcher);
 
     // add *.cpp files to the "files" vector..
     for (std::map<std::string,size_t>::const_iterator it = filemap.begin(); it != filemap.end(); ++it) {
@@ -312,7 +315,7 @@ int main(int argc, char **argv)
 
     // Makefile settings..
     if (release) {
-        makeConditionalVariable(fout, "CXXFLAGS", "-O2 -include lib/cxx11emu.h  -DNDEBUG -Wall");
+        makeConditionalVariable(fout, "CXXFLAGS", "-O2 -include lib/cxx11emu.h -DNDEBUG -Wall -Wno-sign-compare");
     } else {
         // TODO: add more compiler warnings.
         // -Wlogical-op       : doesn't work on older GCC
@@ -332,7 +335,7 @@ int main(int argc, char **argv)
                                 "-Wmissing-declarations "
                                 "-Wmissing-format-attribute "
                                 "-Wno-long-long "
-                                "-Woverloaded-virtual "
+//                                "-Woverloaded-virtual "  // danmar: we get fp when overloading analyseWholeProgram()
                                 "-Wpacked "
                                 "-Wredundant-decls "
                                 "-Wshadow "
@@ -363,7 +366,7 @@ int main(int argc, char **argv)
 
     fout << "BIN=$(DESTDIR)$(PREFIX)/bin\n\n";
     fout << "# For 'make man': sudo apt-get install xsltproc docbook-xsl docbook-xml on Linux\n";
-    fout << "DB2MAN=/usr/share/sgml/docbook/stylesheet/xsl/nwalsh/manpages/docbook.xsl\n";
+    fout << "DB2MAN?=/usr/share/sgml/docbook/stylesheet/xsl/nwalsh/manpages/docbook.xsl\n";
     fout << "XP=xsltproc -''-nonet -''-param man.charmap.use.subset \"0\"\n";
     fout << "MAN_SOURCE=man/cppcheck.1.xml\n\n";
 
@@ -396,8 +399,8 @@ int main(int argc, char **argv)
     fout << "\t./testrunner -g -q\n\n";
     fout << "checkcfg:\tcppcheck\n";
     fout << "\t./test/cfg/runtests.sh\n\n";
-    fout << "dmake:\ttools/dmake.o cli/filelister.o lib/path.o\n";
-    fout << "\t$(CXX) $(CXXFLAGS) -std=c++0x -o dmake tools/dmake.o cli/filelister.o lib/path.o -Ilib $(LDFLAGS)\n";
+    fout << "dmake:\ttools/dmake.o cli/filelister.o cli/pathmatch.o lib/path.o\n";
+    fout << "\t$(CXX) $(CXXFLAGS) -std=c++0x -o dmake tools/dmake.o cli/filelister.o cli/pathmatch.o lib/path.o -Ilib $(LDFLAGS)\n";
     fout << "\t./dmake\n\n";
     fout << "reduce:\ttools/reduce.o externals/tinyxml/tinyxml2.o $(LIBOBJ)\n";
     fout << "\t$(CXX) $(CPPFLAGS) $(CXXFLAGS) -std=c++0x -g -o reduce tools/reduce.o -Ilib -Iexternals/tinyxml $(LIBOBJ) $(LIBS) externals/tinyxml/tinyxml2.o $(LDFLAGS) $(RDYNAMIC)\n\n";
