@@ -32,7 +32,6 @@ class MatchCompiler:
 
     def _reset(self):
         self._rawMatchFunctions = []
-        self._matchStrs = {}
         self._matchFunctionCache = {}
 
     def _generateCacheSignature(
@@ -81,23 +80,13 @@ class MatchCompiler:
 
         self._matchFunctionCache[signature] = id
 
-    def _insertMatchStr(self, look_for):
-        prefix = 'matchStr'
-
-        # Add entry if needed
-        if look_for not in self._matchStrs:
-            pos = len(self._matchStrs) + 1
-            self._matchStrs[look_for] = pos
-
-        return prefix + str(self._matchStrs[look_for])
-
     def _compileCmd(self, tok):
         if tok == '%any%':
             return 'true'
         elif tok == '%bool%':
             return 'tok->isBoolean()'
         elif tok == '%char%':
-            return '(tok->type()==Token::eChar)'
+            return '(tok->tokType()==Token::eChar)'
         elif tok == '%comp%':
             return 'tok->isComparisonOp()'
         elif tok == '%num%':
@@ -107,11 +96,11 @@ class MatchCompiler:
         elif tok == '%op%':
             return 'tok->isOp()'
         elif tok == '%or%':
-            return '(tok->type() == Token::eBitOp && tok->str()==' + self._insertMatchStr('|') + ')/* | */'
+            return '(tok->tokType() == Token::eBitOp && tok->str()==MatchCompiler::makeConstString("|") )'
         elif tok == '%oror%':
-            return '(tok->type() == Token::eLogicalOp && tok->str()==' + self._insertMatchStr('||') + ')/* || */'
+            return '(tok->tokType() == Token::eLogicalOp && tok->str()==MatchCompiler::makeConstString("||"))'
         elif tok == '%str%':
-            return '(tok->type()==Token::eString)'
+            return '(tok->tokType()==Token::eString)'
         elif tok == '%type%':
             return (
                 '(tok->isName() && tok->varId()==0U && !tok->isKeyword())'
@@ -126,7 +115,7 @@ class MatchCompiler:
             print("unhandled:" + tok)
 
         return (
-            '(tok->str()==' + self._insertMatchStr(tok) + ')/* ' + tok + ' */'
+            '(tok->str()==MatchCompiler::makeConstString("' + tok + '"))'
         )
 
     def _compilePattern(self, pattern, nr, varid,
@@ -164,8 +153,7 @@ class MatchCompiler:
 
             # [abc]
             if (len(tok) > 2) and (tok[0] == '[') and (tok[-1] == ']'):
-                ret += '    if (!tok || tok->str().size()!=1U || !strchr("' + tok[
-                    1:-1] + '", tok->str()[0]))\n'
+                ret += '    if (!tok || tok->str().size()!=1U || !strchr("' + tok[1:-1] + '", tok->str()[0]))\n'
                 ret += '        ' + returnStatement
 
             # a|b|c
@@ -200,8 +188,7 @@ class MatchCompiler:
 
             # !!a
             elif tok[0:2] == "!!":
-                ret += '    if (tok && tok->str() == ' + self._insertMatchStr(
-                    tok[2:]) + ')/* ' + tok[2:] + ' */\n'
+                ret += '    if (tok && tok->str() == MatchCompiler::makeConstString("' + tok[2:] + '"))\n'
                 ret += '        ' + returnStatement
                 gotoNextToken = '    tok = tok ? tok->next() : NULL;\n'
 
@@ -272,6 +259,23 @@ class MatchCompiler:
 
         return None
 
+    def _isInString(self, line, pos1):
+        pos = 0
+        inString = False
+        while pos != pos1:
+            if inString:
+                if line[pos] == '\\':
+                    pos += 1
+                elif line[pos] == '"':
+                    inString = False
+            else:
+                if line[pos] == '\\':
+                    pos += 1
+                elif line[pos] == '"':
+                    inString = True
+            pos += 1
+        return inString
+
     def _parseStringComparison(self, line, pos1):
         startPos = 0
         endPos = 0
@@ -505,7 +509,7 @@ class MatchCompiler:
 
         return (
             line[:start_pos] + functionName + str(
-                findMatchNumber) + '(' + tok + more_args + ')' + line[start_pos + end_pos:]
+                findMatchNumber) + '(' + tok + more_args + ') ' + line[start_pos + end_pos:]
         )
 
     def _replaceTokenFindMatch(self, line):
@@ -576,12 +580,13 @@ class MatchCompiler:
 
     def _replaceCStrings(self, line):
         while True:
-            match = re.search('str\(\) *(==|!=) *"', line)
-            if not match:
-                match = re.search('strAt\(.+?\) *(==|!=) *"', line)
+            match = re.search('(==|!=) *"', line)
             if not match:
                 break
 
+            if self._isInString(line, match.start()):
+                break
+
             res = self._parseStringComparison(line, match.start())
             if res is None:
                 break
@@ -589,11 +594,12 @@ class MatchCompiler:
             startPos = res[0]
             endPos = res[1]
             text = line[startPos + 1:endPos - 1]
-            line = line[:startPos] + self._insertMatchStr(text) + line[endPos:]
-
+            line = line[:startPos] + 'MatchCompiler::makeConstStringBegin' + text + 'MatchCompiler::makeConstStringEnd' + line[endPos:]
+        line = line.replace('MatchCompiler::makeConstStringBegin', 'MatchCompiler::makeConstString("')
+        line = line.replace('MatchCompiler::makeConstStringEnd', '")')
         return line
 
-    def convertFile(self, srcname, destname):
+    def convertFile(self, srcname, destname, line_directive):
         self._reset()
 
         fin = open(srcname, "rt")
@@ -602,6 +608,7 @@ class MatchCompiler:
 
         header = '#include "token.h"\n'
         header += '#include "errorlogger.h"\n'
+        header += '#include "matchcompiler.h"\n'
         header += '#include <string>\n'
         header += '#include <cstring>\n'
         # header += '#include <iostream>\n'
@@ -619,36 +626,22 @@ class MatchCompiler:
 
             code += line
 
-        # Compute string list
-        stringList = ''
-        for match in sorted(self._matchStrs, key=self._matchStrs.get):
-            stringList += 'static const std::string matchStr' + \
-                str(self._matchStrs[match]) + '("' + match + '");\n'
-
         # Compute matchFunctions
         strFunctions = ''
         for function in self._rawMatchFunctions:
             strFunctions += function
 
+        lineno = ''
+        if line_directive:
+            lineno = '#line 1 "' + srcname + '"\n'
+
         fout = open(destname, 'wt')
-        fout.write(header + stringList + strFunctions + code)
+        fout.write(header + strFunctions + lineno + code)
         fout.close()
 
 
 def main():
     # Main program
-    build_dir = 'build'
-
-    # Check if we are invoked from the right place
-    if not os.path.exists('lib') and not os.path.exists('samples'):
-        print('Please invoke from the top level cppcheck source dir. Example: tools/matchcompiler.py')
-        sys.exit(-1)
-
-    # Create build directory if needed
-    if not os.path.exists(build_dir):
-        os.makedirs(build_dir)
-    if not os.path.isdir(build_dir):
-        raise Exception(build_dir + ' is not a directory')
 
     # Argument handling
     parser = argparse.ArgumentParser(
@@ -657,15 +650,48 @@ def main():
                         help='verify compiled matches against on-the-fly parser. Slow!')
     parser.add_argument('--show-skipped', action='store_true', default=False,
                         help='show skipped (non-static) patterns')
+    parser.add_argument('--read-dir', default="lib",
+                        help='directory from which files are read')
+    parser.add_argument('--write-dir', default="build",
+                        help='directory into which files are written')
+    parser.add_argument('--prefix', default="",
+                        help='prefix for build files')
+    parser.add_argument('--line', action='store_true', default=False,
+                        help='add line directive to input files into build files')
+    parser.add_argument('file', nargs='*',
+                        help='file to complile')
     args = parser.parse_args()
+    lib_dir = args.read_dir
+    build_dir = args.write_dir
+    line_directive = args.line
+    files = args.file
+
+    # Check if we are invoked from the right place
+    if not os.path.exists(lib_dir):
+        print('Directory "' + lib_dir + '"not found.')
+        sys.exit(-1)
+
+    # Create build directory if needed
+    if not os.path.exists(build_dir):
+        os.makedirs(build_dir)
+    if not os.path.isdir(build_dir):
+        raise Exception(build_dir + ' is not a directory')
 
     mc = MatchCompiler(verify_mode=args.verify,
                        show_skipped=args.show_skipped)
 
-    # convert all lib/*.cpp files
-    for f in glob.glob('lib/*.cpp'):
-        print(f + ' => ' + build_dir + '/' + f[4:])
-        mc.convertFile(f, build_dir + '/' + f[4:])
+    if not files:
+        # select all *.cpp files in lib_dir
+        for f in glob.glob(lib_dir + '/*.cpp'):
+            files.append(f[len(lib_dir) + 1:])
+
+    # convert files
+    for fi in files:
+        pi = lib_dir + '/' + fi
+        fo = args.prefix + fi
+        po = build_dir + '/' + fo
+        print(pi + ' => ' + po)
+        mc.convertFile(pi, po, line_directive)
 
 if __name__ == '__main__':
     main()
